dnl Usage:
dnl AC_CHECK_ODBC
dnl Sets ODBC_LIBS and ODBC_INCLUDES

AC_DEFUN([AC_CHECK_ODBC],
[
  AC_ARG_WITH(
    unixodbc-prefix, 
    AC_HELP_STRING(
      [--with-unixodbc-prefix=DIR],
      [root of the unixODBC installation]
    ),
    [ac_unixodbc_prefix=$withval],
    [ac_unixodbc_prefix=""]
  )
  
  if test -z "$ac_unixodbc_prefix" ; then
    AC_CHECK_LIB(odbc, SQLDataSources, ODBC_LIB=-lodbc,
                 AC_MSG_ERROR("No odbc library found!"))

    AC_CHECK_LIB(odbcinst, SQLReadFileDSN, ODBCINST_LIB=-lodbcinst,
                 AC_MSG_ERROR("No odbcinst library found!"))

    ODBC_LIBS="$ODBC_LIB $ODBCINST_LIB"
    ODBC_INCLUDES=""
  else
    if test "x$host_cpu" = "xx86_64"; then
      if test -e "$ac_unixodbc_prefix/lib64"; then
        ODBC_LIBS="-L$ac_unixodbc_prefix/lib64 -lodbc -lodbcinst"
      else
        ODBC_LIBS="-L$ac_unixodbc_prefix/lib -lodbc -lodbcinst"
      fi
    else
      ODBC_LIBS="-L$ac_unixodbc_prefix/lib -lodbc -lodbcinst"
    fi
    ODBC_INCLUDES="-I$ac_unixodbc_prefix/include"

    ac_save_cppflags=$CPPFLAGS
    ac_save_libs=$LIBS
    CPPFLAGS="$CPPFLAGS $ODBC_INCLUDES"
    LIBS="$LIBS $ODBC_LIBS"

    AC_MSG_CHECKING([Checking for unixODBC])
    AC_TRY_LINK([#include<sql.h>],
		[SQLHSTMT st;
		 SQLCancel(st);],
		[ac_have_odbc=yes],
		[ac_have_odbc=no])
    AC_MSG_RESULT([$ac_have_odbc])

    if test "x$ac_have_odbc" = "xno" ; then
       AC_MSG_ERROR([No odbc library found!])
    fi

    CPPFLAGS=$ac_save_cppflags
    LIBS=$ac_save_libs
  fi

  AC_SUBST(ODBC_LIBS)
  AC_SUBST(ODBC_INCLUDES) 
])
 
