/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/*
 * AMGAFrame.h
 *
 *  Created on: Mar 23, 2009
 *      Author: root
 */

#ifndef AMGAFRAME_H_
#define AMGAFRAME_H_

#include "server/MDInterpreter.h"
#include "server/MDStandalone.h"
#include "server/MDServerFactory.h"
#include "server/MDSocketBuffer.h"
#include "server/MDConManMem.h"
#include "server/MDConManDB.h"
#include "server/UserContext.h"
#include "server/VOMSInfo.h"
#include "server/protocolHandling.h"
#include "replicationdaemon/MDLogWriter.h"
#include "replicationdaemon/LogReceiver.h"
#include "replicationdaemon/MountManager.h"
#include "replicationdaemon/ReplicationManager.h"

#include "util/DatabaseConnection.h"
#include "util/SecurityUtils.h"
#include "util/Socket.h"
#include "util/sslHelpers.h"
#include "util/amgaProtocol.h"

#include "configparser/ConfigParser.h"

#include <cstdio>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>
#include <getopt.h>
#include <time.h>
#include <assert.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <openssl/ssl.h>
#include <openssl/evp.h>
#include <openssl/err.h>
#include <openssl/safestack.h>

#include <iostream>
#include <list>
#include <sstream>
#include <pwd.h>

#include "debug.h"

using namespace std;



class AMGAFrame {

public :

    AMGAFrame() ;
     virtual ~AMGAFrame() ;

     virtual int slaveProcess(TCPServerSocket &serverSocket) =0 ;
          virtual int returnStatistics(TCPSocket &socket) = 0;

protected:

	// for callback and signal functions
	static AMGAFrame* pAMGAFrame;

	// signal handler variables
	static sighandler_t oldSIGINT;
	static sighandler_t oldSIGCHILD;

	static bool debug;

	int initSessions(SSL_CTX *ctx, char *appData, int sessionTimeout);

	void removeDead();

	int configLoad();
	int initialize();
	void MainLoop();

	long calclock(struct timespec *myclock);
	int initConnection(TCPSocket &socket, MDServer *connection, UserContext &userCtx);



	MDConMan *conMan;

	int port;
	static string logFile;
	string runningMode; // KIMNG090323 the mode of AMGA - Process Mode or Thread Mode.  the default value is "process"
	bool daemon;
	/*
	 static bool forceSessions = false;
	 static bool allowSessions = true;
	 static int maxConnects = 1000000;
	 static long sessionTimeout = 300;
	 static int execUserID = -1;
	 static int execUserGroup = -1;
	 static std::string execRoot;
	 static int maxConnectsPerUser = -1;
	 static int reservedConnections = -1;
	 */
	bool forceSessions;
	bool allowSessions;
	int maxConnects;
	long sessionTimeout;
	int execUserID;
	int execUserGroup;
	std::string execRoot;
	int maxConnectsPerUser;
	int reservedConnections;
	unsigned int maxProcesses; // Initialize connection management
	// Init session cache
	int maxSessionSize;
	int maxSessions;
	int minProcesses;
	int idleTimeout;
	//static bool requireAuthentication = true;
	//static bool allowPasswordAuthentication = false;
	//static bool allowCertAuth=false;
	//static bool allowMyProxyHack = false;

	ConfigParser configParser;

	// Inititalize options to defaults
	static std::ofstream logStream;

	// Connection management: Used from within interrupt, no STL!
	int numSlots;
	int *children;
	int *deadChildren;

	int threadNum;

	// KIMNG090403 without static
	AuthContext authCtx;
	volatile bool terminateRequest;

public:

	void init(const ConfigParser& config, string strRunningMode, bool bDebug, bool daemon, int nPort, string strLogFile);
	int run();

	static int ssl_callback_NewSessionCacheEntry(SSL *ssl, SSL_SESSION *session);
	static SSL_SESSION *ssl_callback_GetSessionCacheEntry(SSL *ssl, unsigned char *sessionID, int idLen, int *doCopy);
	static void ssl_callback_RemoveSessionCacheEntry(SSL_CTX *ctx, SSL_SESSION *session);
	static void sig_handler(int signo);
	static void child_sig_handler(int sig);
	static void int_sig_handler(int sig);
        static void hup_sig_handler(int sig);
	static void child_term_handler(int sig);
};



#endif /* AMGAFRAME_H_ */
