/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/*
 * ProcessFrame.cpp
 *
 *  Created on: Mar 27, 2009
 *      Author: root
 */

#include "ProcessFrame.h"


ProcessFrame::ProcessFrame() {
	// TODO Auto-generated constructor stub
}


ProcessFrame::~ProcessFrame() {
	// TODO Auto-generated destructor stub
}

/* Change Architecture */
int ProcessFrame::slaveProcess(TCPServerSocket &serverSocket) {

	struct sigaction act;

	memset(&act, 0, sizeof(act));
	act.sa_handler = SIG_IGN;
	sigaction(SIGINT, &act, NULL);

	signal(SIGTERM, AMGAFrame::child_term_handler);
	siginterrupt(SIGTERM, 1);

	/* Change Architecture */
	// Tell that we are up and running, but not yet finished
	conMan->timestamp(2);

	// Initialize connection to backend
	MDSocketBuffer rwBuffer;
	MDServer *connection = NULL;
	try {
		connection = MDServerFactory::mdServer(rwBuffer);
		if (!connection || !connection->isConnected()) {
			ERRLOG("Could not connect to DB" << std::endl);
			return -1;
		}
	} catch (SQLException &ex) {
		ERRLOG("Could not connect to DB" << std::endl);
		return -1;
	}

	connection->setDebug(debug);
	connection->printODBCDriverVersion(); // Debug printout

	/* Change Architecture */
	connection->setConMan(conMan);

	// Initialize User Manager
	std::list<MDUserManager *> userManagers;
	createUserManagers(*(ConfigParser::configParser), userManagers);
	connection->initializeUserManager(userManagers);

	// We will handle up to maxConnects connections, then we will die
	for (int i = 0; i < maxConnects && !terminateRequest; i++) {
		DMESG("Process does accept" << std::endl);
		/* Change Architecture */
		conMan->timestamp(3);
		// We will handle up to maxConnects connections, then we will die

		try {
			auto_ptr<TCPSocket> socket(serverSocket.accept());
			DMESG("ACCEPT" << std::endl);
			/* Change Architecture */
			conMan->timestamp(4);
			conMan->notifyMaster();
			rwBuffer.setSocket(socket.get());

			/* Change Architecture */
			handleConnection(*socket, connection);

                        /* federation */
                        if( connection->mdfed != NULL ) {
                          connection->mdfed->disconnectFed(false);
                          delete connection->mdfed;
                          connection->mdfed = NULL;
                        }

		} catch (SocketException &ex) {
			if (errno!=EINTR) {
				PERRLOG("Error accepting connection: " << ex.what() << endl);
			}
			break;
		}
	}
	delete connection;

	removeUserManagers(userManagers);

	LOG("Finished serving, going away" << std::endl);
	return 0;
}

int ProcessFrame::handleConnection(TCPSocket &socket, MDServer *connection) {

	UserContext userCtx;

	int ret = initConnection(socket, connection, userCtx);
	DMESG("-------------------- ret : " << ret << " in handleConnection " << endl);
	if (ret == -1) {
		// Error
		return -1;
	} else if (ret == 1) {
		// Ok
		return 0;
	}

	MDInterpreter interpreter;

	// From now on, use TCP_CORK ing
	//  sockflag=0;
	//  setsockopt(ipConnection, IPPROTO_TCP, TCP_NODELAY, &sockflag, sizeof(sockflag));
	//  sockflag=1;
	//  setsockopt(ipConnection, IPPROTO_TCP, TCP_CORK, &sockflag, sizeof(sockflag));

	// Loops as long as commands are sent until quit
	std::string response;
	// A priory, the session is not to be stored, unless suspendConnections is
	// true and the sessionID has been sent to the client
	bool storeSession = false;
	while (1) {
//		bool transaction = false;

		DMESG("Listening..." << std::endl);
		response = "";
		connection->getBuffer().start();

		LOG("Before readLine" << std::endl);

		std::string line;
		try {
			line = socket.readLine();
			AMGA::decodeLine(line);
		} catch (SocketException &ex) {
			DMESG("Client disconnect: " << ex.what() << endl);
			break;
		}

		// TO-DO KIMNG
		conMan->timestamp(5, userCtx.login);
		LOG(userCtx.login << "> " << line.substr(0,200) << std::endl);
		MDInterpreter::CommandToken c = interpreter.parseCommand(line);

		// Special commands
		// Exit immediately
		if (c == MDInterpreter::cExit)
			break;
		if (c == MDInterpreter::cClose) {
			if (allowSessions) {
				storeSession = true;
				break;
			} else {
				break; // handle as if exit
			}
		}

		// Ignore commands which are out of context
		if (c == MDInterpreter::cPut)
			continue;

		// Set the next command to be executed.
		// This is ignored if replication is disabled.
		connection->setNextCommand(line);

		try {
			interpreter.execLast(*connection);

/*
                        if ( connection->dbConn->connAlive == false) {
                           connection->getBuffer().start();
                           connection->getBuffer().append("901 DB Connection is broken\n");
                        }
*/

			connection->getBuffer().flush();
		} catch (exception &ex) {
			// Don't do anything with the exception
		}

		if (c == MDInterpreter::cUpload) {
			bool failed = false;
			while (c != MDInterpreter::cAbort && c != MDInterpreter::cCommit) {
				try {
					line = socket.readLine();
					AMGA::decodeLine(line);
				} catch (SocketException &ex) {
					DMESG("Client disconnect: " << ex.what() << endl);
					break;
				}

				DMESG("Received command: >" << line.substr(0,200) << "<" << std::endl);
				c = interpreter.parseCommand(line);
				if (c == MDInterpreter::cExit)
					break;
				if (failed) {
					// Skip all commands until the client stops sending more data
					continue;
				}

				if (c != MDInterpreter::cAbort && c != MDInterpreter::cCommit
						&& c != MDInterpreter::cPut) {
					DMESG("Command not allowed\n");
					connection->getBuffer().append("9 Command not allowed\n");
					failed = true;
				} else {
					// Must wait for the previous command to be fully executed
					// before setting the values on these structures, since they
					// are shared.
					connection->setNextCommand(line);

					if (interpreter.execLast(*connection)) {
						DMESG("Command failed\n");
						failed = true;
					}
				}
			}
			connection->getBuffer().flush();DMESG("Streaming master exits loop" << std::endl);
			if (c == MDInterpreter::cExit)
				break;DMESG("Streaming master FINISH" << std::endl);
		}

		if (connection->getBuffer().isCancelled()) {
			DMESG("Request was cancelled\n");
			response.append("\030");
			response.append("12 Request cancelled\n");
		}
		//    if(transmitError) // Don't send anything more if transmit error
		//      break;

		if (connection->transactionInProgress() || (!forceSessions
				&& !storeSession)) {
			// Send EOT to signal end of response
			DMESG("Sending EOT" << std::endl);
			response.append("\004\004"); // Append final EOT char
			try {
				socket.send(response);
			} catch (SocketException &ex) {
				ERRLOG("Cannot send response: " << ex.what() << endl);
				sslPrintErrors();
				//        transmitError=true;
				break;
			}
		} else {
			storeSession = true;
			break;
		}
	} // End of loop over all client commands

	DMESG("Closing connection (pid " << getpid() << ")\n");
	if (storeSession) {
		// Retrieve settings from connection and store them in context
		userCtx.currentDir = connection->getDir();

		DMESG("Saving session" << std::endl);
		uint64_t sessionID = conMan->saveSessionUser(userCtx);
		if (!sessionID)ERRLOG("Could not save session " << sessionID << std::endl);

		DMESG("Suspending session " << sessionID << std::endl);
		char sBuffer[60];
		sprintf(sBuffer, "\004session%llu\004", sessionID);
		response.append(sBuffer);
		storeSession = true;

		// Send EOT to signal end of response
		//    DMESG("Sending EOT: " << response << std::endl);
		DMESG("Sending EOT:\n");
		try {
			socket.send(response);
		} catch (SocketException &ex) {
			ERRLOG("Cannot send response");
			sslPrintErrors();
		}
	}

	int count = 1000;

	while (connection->transactionInProgress() && count--)
		connection->transactionAbort();

	return 0;
}

/* Change Architecture */
int ProcessFrame::returnStatistics(TCPSocket &socket) {
	std::ostringstream statistics;
	StatsCollector::dumpAsXml(statistics);
	try {
		DMESG("" << statistics.str() << endl);
		socket.send(statistics.str());
	} catch (SocketException &ex) {
		PERRLOG("Cannot send response\n");
		return -1;
	}DMESG("Statistics sent" << std::endl);
	return 0;
}

