/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/*
 * ThreadFrame.cpp
 *
 *  Created on: Mar 27, 2009
 *      Author: root
 */

#include "ThreadFrame.h"

ThreadFrame::ThreadFrame() {
        // TODO Auto-generated destructor stub
}

ThreadFrame::~ThreadFrame() {
	// TODO Auto-generated destructor stub
}


/* Change Architecture */
int ThreadFrame::slaveProcess(TCPServerSocket &serverSocket) {


	DMESG("SlaveProcess in ThreadFrame\n");
	struct sigaction act;

	memset(&act, 0, sizeof(act));
	act.sa_handler = SIG_IGN;
	sigaction(SIGINT, &act, NULL);

	signal(SIGTERM, child_term_handler);
	siginterrupt(SIGTERM, 1);

		/* Change Architecture */
	// Tell that we are up and running, but not yet finished
	conMan->timestamp(2);

	LOG("Thread Number : " << threadNum << endl);

	//int status;
	int threadNum = AMGAFrame::threadNum;
	pthread_t thread_id[threadNum];

	// KIMNG080418 create thread and initialize thread's variables with MDServer instance
	threadMan = new MDThreadMan();

	// KIMNG080514 create a DB Connection
	try {
		threadMan->setDBConnection(DatabaseConnection::openConnection());DMESG("threadMan->getDBConnection()->getODBCDriverName() : " << threadMan->getDBConnection()->getODBCDriverName() << std::endl);
	} catch (SQLException &ex) {
		ERRLOG("Could not connect to DB" << std::endl);
		return -1;
	}

	// initialize threadMan
	if (threadMan->init(threadNum, port) < 0) {
		PERRLOG("error : failure of initializing Thread Mangement Instance"<< std::endl);
		return -1;
	}

	// KIMNG080516 set a serverSocket in threadMan to use it in threads
	threadMan->setServerSock(&serverSocket);
	int tasknum[threadNum];
	for (int i = 0; i < threadNum; i++) {
		tasknum[i] = i;
	}

	// create threads to prefork
	for (int i = 0; i < threadNum; i++) {
		//int create_result = pthread_create(&thread_id[i], NULL, ThreadFrame::clientHandleThread, (void *) &tasknum[i]);
		threadIndex = tasknum[i];
		int create_result = pthread_create(&thread_id[i], NULL, ThreadFrame::clientHandleThread, (void *)this);
		if (create_result < 0) {
			// logging error for creating thread
			PERRLOG("Thread create error : [" << i << "]"<< std::endl);
			return -1;
		}DMESG("create thread [" << tasknum[i] << "] : success "<< std::endl);
	}

	void *ret;
	for (int j = 0; j < threadNum; j++) {
		pthread_join(thread_id[j], &ret);
	}
	LOG("Finished serving, going away" << std::endl);
	return 0;
}



void ThreadFrame::run() {

	struct timespec req;
	req.tv_sec = 0;
	req.tv_nsec = 1;

	// Open the ServerSocket
	//int threadIndex = *((int *) index);
	// TO-DO : signal process
	DMESG("Thread[" << threadIndex << "] : Start" << std::endl);

	// Initialize connection to backend
	MDSocketBuffer rwBuffer;
	MDServer *connection = NULL;

	// KIMNG080507 get mutext
	if (threadMan->getDBLock(threadIndex) < 0) {
		threadMan->setStatus(threadIndex, 0);

		PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl);
		pthread_exit((void *) NULL);
	}
	// KIMNG080515 Get the DB Connection instance to share when creating thread's own MDServer instance
	try {
		connection = MDServerFactory::mdServer(rwBuffer,
				threadMan->getDBConnection());
		if (!connection || !connection->isConnected()) {
			ERRLOG("Could not connect to DB" << std::endl);
			pthread_exit((void *) NULL);
		}
	} catch (SQLException &ex) {
		ERRLOG("Could not connect to DB" << std::endl);
		pthread_exit((void *) NULL);
	}

	// KIMNG080507 release mutext
	if (threadMan->releaseDBLock(threadIndex) < 0) {
		PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
		threadMan->setStatus(threadIndex, 0);
		pthread_exit((void *) NULL);
	}

	// KIMNG080515 These codes related with a DB Connection instance is used when threads are created.
	// Therefore it is impossible that threads call these initial codes for MDServer concurrently.
	connection->setDebug(debug);

	// Initialize User Manager
	std::list<MDUserManager *> userManagers;
	createUserManagers(*(ConfigParser::configParser), userManagers);
	connection->initializeUserManager(userManagers); // Using dbConn to set a DB Connection for User Management

	// We will handle up to maxConnects connections, then we will die
	for (int i = 0; i < maxConnects && !terminateRequest; i++) {

		conMan->timestamp(3); // KIMNG090304 DB Connection is connected and the time before accept

		try {

			// KIMNG080507 get mutext for accept
			if (threadMan->getAcceptLock(threadIndex) < 0) {
				threadMan->setStatus(threadIndex, 0);
				PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
				pthread_exit((void *) NULL);
			}

			TCPServerSocket *serverSocket = threadMan->getServerSock();
			auto_ptr<TCPSocket> socket(serverSocket->accept());
			DMESG("PID[" << getpid() << "]Thread[" << threadIndex << "] : ACCEPT" << std::endl);

			// KIMNG080507 release mutext for accept
			if (threadMan->releaseAcceptLock(threadIndex) < 0) {
				PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
				threadMan->setStatus(threadIndex, 0);
				pthread_exit((void *) NULL);
			}
			conMan->timestamp(4);
			conMan->notifyMaster();

			rwBuffer.setSocket(socket.get());

			// change status of the thread.
			threadMan->setStatus(threadIndex, 2); // The start of a thread

			DMESG("Thread[" << threadIndex << "] : Ready to handle client's request" << std::endl);

			handleConnection(*socket, connection, threadIndex);

                        /* federation */
                        if( connection->mdfed != NULL ) {
                          connection->mdfed->disconnectFed(false);
                          delete connection->mdfed;
                          connection->mdfed = NULL;
                        }

			//DMESG("PID[" << getpid() << "]Thread[" << threadIndex << "] - DONE" << std::endl);

			DMESG("Thread[" << threadIndex << "] : Finish handlling client's request" << std::endl);

		} catch (SocketException &ex) {
			LOG(ex.what() << std::endl);
		}

		nanosleep(&req, NULL); // KIMNG0080502 this line is essential to make thread's run right!!!s

	}

        removeUserManagers(userManagers);

	// KIMNG 080625 delete MDServer instance
	delete connection;
}

int ThreadFrame::handleConnection(TCPSocket &socket, MDServer *connection, int threadIndex) {

	// KIMNG080526 The flag to check if a thread holds a lock
	bool isGetLock = false;

	UserContext userCtx;

	// KIMNG080507 get mutext
	if (!isGetLock) {
		if (threadMan->getDBLock(threadIndex, 3) < 0) {
			threadMan->setStatus(threadIndex, 0);
			PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl);
			return -1;
		}
		isGetLock = true;
	}

	// KIMNG080515 get the DB Connection instance to share
	connection->setDBConnection(threadMan->getDBConnection(), false);

	int ret = initConnection(socket, connection, userCtx);

	// KIMNG080507 release mutext
	if (threadMan->releaseDBLock(threadIndex, 3) < 0) {
		PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
		threadMan->setStatus(threadIndex, 0);
	}
	isGetLock = false;

	DMESG("-------------------- ret : " << ret << " in handleConnection " << endl);
	if (ret == -1) {
		// Error
		return -1;
	} else if (ret == 1) {
		// Ok
		return 0;
	}

	MDInterpreter interpreter;

	// From now on, use TCP_CORK ing
	//  sockflag=0;
	//  setsockopt(ipConnection, IPPROTO_TCP, TCP_NODELAY, &sockflag, sizeof(sockflag));
	//  sockflag=1;
	//  setsockopt(ipConnection, IPPROTO_TCP, TCP_CORK, &sockflag, sizeof(sockflag));

	// Loops as long as commands are sent until quit
	std::string response;
	// A priory, the session is not to be stored, unless suspendConnections is
	// true and the sessionID has been sent to the client
	bool storeSession = false;
	while (1) {

		DMESG("Listening..." << std::endl);
		response = "";
		connection->getBuffer().start();

		std::string line;
		try {
			line = socket.readLine();
			AMGA::decodeLine(line);
		} catch (SocketException &ex) {
			DMESG("Client disconnect: " << ex.what() << endl);
			break;
		}

		// TO-DO KIMNG
		conMan->timestamp(5, userCtx.login);
		LOG(userCtx.login << "> " << line << std::endl);
		MDInterpreter::CommandToken c = interpreter.parseCommand(line);

		// Special commands
		// Exit immediately
		if (c == MDInterpreter::cExit)
			break;
		if (c == MDInterpreter::cClose) {
			if (allowSessions) {
				storeSession = true;
				break;
			} else {
				break; // handle as if exit
			}
		}

		// Ignore commands which are out of context
		if (c == MDInterpreter::cPut)
			continue;
		// KIMNG080507 get mutext
		if (!isGetLock) {
			if (threadMan->getDBLock(threadIndex, 1) < 0) {
				threadMan->setStatus(threadIndex, 0);
				PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
				break;
			}

			isGetLock = true;
		}

		// KIMNG080515 get the DB Connection instance to share
		connection->setDBConnection(threadMan->getDBConnection(), false);
		// Set the next command to be executed.
		// This is ignored if replication is disabled.
		connection->setNextCommand(line);

		try {
			interpreter.execLast(*connection);

/*
                        if ( connection->dbConn->connAlive == false ) {
                          connection->getBuffer().start();
                          connection->getBuffer().append("901 DB Connection is broken\n");
                         }
*/

			connection->getBuffer().flush();
		} catch (exception &ex) {
			// Don't do anything with the exception
		}
		// KIMNG080507 release mutext
		if (!connection->transactionInProgress()) {
			if (threadMan->releaseDBLock(threadIndex, 1) < 0) {
				PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
				threadMan->setStatus(threadIndex, 0);
				break;
			}

			isGetLock = false;
		}

		if (c == MDInterpreter::cUpload) {
			bool failed = false;
			while (c != MDInterpreter::cAbort && c != MDInterpreter::cCommit) {
				try {
					line = socket.readLine();
					AMGA::decodeLine(line);
				} catch (SocketException &ex) {
					DMESG("Client disconnect: " << ex.what() << endl);
					break;
				}

				DMESG("Received command: >" << line << "<" << std::endl);
				c = interpreter.parseCommand(line);
				if (c == MDInterpreter::cExit)
					break;
				if (failed) {
					// Skip all commands until the client stops sending more data
					continue;
				}

				if (c != MDInterpreter::cAbort && c != MDInterpreter::cCommit
						&& c != MDInterpreter::cPut) {
					DMESG("Command not allowed\n");
					connection->getBuffer().append("9 Command not allowed\n");
					failed = true;
				} else {
					// Must wait for the previous command to be fully executed
					// before setting the values on these structures, since they
					// are shared.
					connection->setNextCommand(line);

					if (interpreter.execLast(*connection)) {
						DMESG("Command failed\n");
						failed = true;
					}
					// KIMNG080507 release mutext
					if (!connection->transactionInProgress()) {
						if (threadMan->releaseDBLock(threadIndex, 2) < 0) {
							PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
							threadMan->setStatus(threadIndex, 0);
							break;
						}
						isGetLock = false;
					}
				}
			}
			connection->getBuffer().flush();DMESG("Streaming master exits loop" << std::endl);
			if (c == MDInterpreter::cExit)
				break;DMESG("Streaming master FINISH" << std::endl);
		}

		if (connection->getBuffer().isCancelled()) {
			DMESG("Request was cancelled\n");
			response.append("\030");
			response.append("12 Request cancelled\n");
		}
		//    if(transmitError) // Don't send anything more if transmit error
		//      break;

		if (connection->transactionInProgress() || (!forceSessions
				&& !storeSession)) {
			// Send EOT to signal end of response
			DMESG("Sending EOT" << std::endl);
			response.append("\004\004"); // Append final EOT char
			try {
				socket.send(response);
			} catch (SocketException &ex) {
				ERRLOG("Cannot send response: " << ex.what() << endl);
				sslPrintErrors();
				//        transmitError=true;
				break;
			}
		} else {
			storeSession = true;
			break;
		}
	} // End of loop over all client commands

	DMESG("Closing connection (pid " << getpid() << ")\n");
	if (storeSession) {
		// Retrieve settings from connection and store them in context
		userCtx.currentDir = connection->getDir();

		DMESG("Saving session" << std::endl);
		uint64_t sessionID = conMan->saveSessionUser(userCtx);
		if (!sessionID)ERRLOG("Could not save session " << sessionID << std::endl);

		DMESG("Suspending session " << sessionID << std::endl);
		char sBuffer[60];
		sprintf(sBuffer, "\004session%llu\004", sessionID);
		response.append(sBuffer);
		storeSession = true;

		// Send EOT to signal end of response
		//    DMESG("Sending EOT: " << response << std::endl);
		DMESG("Sending EOT:\n");
		try {
			socket.send(response);
		} catch (SocketException &ex) {
			ERRLOG("Cannot send response");
			sslPrintErrors();
		}
	}

	int count = 1000;

	// KIMNG080507 get mutext
	if (!isGetLock) {
		if (threadMan->getDBLock(threadIndex, 3) < 0) {
			threadMan->setStatus(threadIndex, 0);
			PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl);
			return -1;
		}
		isGetLock = true;
	}

	// KIMNG080515 get the DB Connection instance to share
	connection->setDBConnection(threadMan->getDBConnection(), false);

	while (connection->transactionInProgress() && count--)
		connection->transactionAbort();

	// KIMNG080507 release mutext
	if (!connection->transactionInProgress()) {
		if (threadMan->releaseDBLock(threadIndex, 3) < 0) {
			PERRLOG("error : Failure of initializing the pthread's mutex variable "<< std::endl)
			threadMan->setStatus(threadIndex, 0);
		}
		isGetLock = false;
	}
	return 0;
}

/* Change Architecture */
int ThreadFrame::returnStatistics(TCPSocket &socket) {
	std::ostringstream statistics;
	StatsCollector::dumpAsXml(statistics);
	try {
		DMESG("" << statistics.str() << endl);
		socket.send(statistics.str());
	} catch (SocketException &ex) {
		PERRLOG("Cannot send response\n");
		return -1;
	}DMESG("Statistics sent" << std::endl);
	return 0;
}
