/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// $Id: amgad.cc,v 1.35 2010/10/26 07:27:47 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "configparser/ConfigParser.h"

#include <cstdio>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <iostream>
#include <list>
#include <sstream>

#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

#include "debug.h"
#include "AMGAFrame.h"
#include "ProcessFrame.h"
#include "ThreadFrame.h"

// for ApMon codes
#include <unistd.h>

#include <stdlib.h>
#include <time.h>
#include <netdb.h>
#include "ApMon/ApMon.h"
#include "ApMon/utils.h"
#include "string.h"
#include "stdio.h"
#include "ApMon/proc_utils.h"
#include "ApMon/mon_constants.h"
#include <iostream>
#include <fstream>

using namespace apmon_utils;

static struct option options[] = { { "help", 0, 0, 'h' },
		{ "debug", 0, 0, 'd' }, { "daemon", 0, 0, 'D' }, { "port", 1, 0, 'p' },
		{ "config-file", 1, 0, 'c' }, { "log", 1, 0, 'l' }, { 0, 0, 0, 0 } };

void printUsage() {
	std::cout << "Usage: amgad [options]" << std::endl;
	std::cout << "Options:  -p <port>        : server port" << std::endl;
	std::cout << "          -d               : debug on" << std::endl;
	std::cout << "          -D               : deamon mode" << std::endl;
	std::cout << "          -c <configFile>  : config file" << std::endl;
	std::cout << "          -l <logFile>     : log file" << std::endl;
	std::cout << "          -H               : Allow holding connections"
			<< std::endl;
}

typedef struct {
	string frequency;
	string vendor;
	string modelName;
	string cpuNums;
} CPUINFO;

typedef struct {
	string totalMem;
	string swapMem;
} MEMINFO;

int getNumCPUs() throw(procutils_error) {
#ifdef WIN32
	return 0;
#else
	int numCPUs = 0;
	char line[MAX_STRING_LEN];

	FILE *fp = fopen("/proc/stat", "r");

	if (fp == NULL)
		throw procutils_error("[ getNumCPUs() ] Could not open /proc/stat.");

	while (fgets(line, MAX_STRING_LEN, fp)) {
		if (strstr(line, "cpu") == line && isdigit(line[3]))
			numCPUs++;
	}

	fclose(fp);
	return numCPUs;
#endif
}

void getCPUInfo(CPUINFO *cpuInfo) throw(procutils_error) {
#ifndef WIN32
	double freq = 0;
	char line[MAX_STRING_LEN], s1[100], s2[100], s3[100];
	//  char buf[MAX_STRING_LEN];
	//  char *pbuf = buf;
	char *tmp, *tmp_trim;
	bool freqFound = false, bogomipsFound = false;

	FILE *fp = fopen("/proc/cpuinfo", "r");
	if (fp == NULL)
		throw procutils_error("[ getCPUInfo() ] Could not open /proc/cpuinfo");

	while (fgets(line, MAX_STRING_LEN, fp)) {
		if (strstr(line, "cpu MHz") == line) {
			sscanf(line, "%s %s %s %lf", s1, s2, s3, &freq);
			char temp[256] = { 0x00, };
			sprintf(temp, "%f", freq);
			cpuInfo->frequency = temp;
			freqFound = true;
			continue;
		}

		if (strstr(line, "vendor_id") == line) {
			tmp = strtok/*_r*/(line, ":");//, &pbuf);
			/* take everything that's after the ":" */
			tmp = strtok/*_r*/(NULL, ":");//, &pbuf);
			tmp_trim = trimString(tmp);
			cpuInfo->vendor = tmp_trim;
			// strcpy(apm.cpuVendor, tmp_trim);
			free(tmp_trim);
			continue;
		}

		if (strstr(line, "model name") == line) {
			tmp = strtok/*_r*/(line, ":");//, &pbuf);
			/* take everything that's after the ":" */
			tmp = strtok/*_r*/(NULL, ":");//, &pbuf);
			tmp_trim = trimString(tmp);
			cpuInfo->modelName = tmp_trim;
			//strcpy(apm.cpuModelName, tmp_trim);
			free(tmp_trim);
			continue;
		}
	}
	fclose(fp);
	if (!freqFound)
		throw procutils_error(
				"[ getCPUInfo() ] Could not find frequency or bogomips in /proc/cpuinfo");
#endif
}

void getSysMem(MEMINFO *memInfo) throw(procutils_error) {
#ifndef WIN32
	char s1[20], line[MAX_STRING_LEN];
	bool memFound = false, swapFound = false;
	char valMem[255] = { 0x00, }, valSwap[255] = { 0x00, };
	FILE *fp1;

	fp1 = fopen("/proc/meminfo", "r");
	if (fp1 == NULL)
		throw procutils_error("[ getSysMem() ] Could not open /proc/meminfo");

	while (fgets(line, MAX_STRING_LEN, fp1)) {
		if (strstr(line, "MemTotal:") == line) {
			sscanf(line, "%s %s", s1, valMem);
			memFound = true;
			continue;
		}

		if (strstr(line, "SwapTotal:") == line) {
			sscanf(line, "%s %s", s1, valSwap);
			swapFound = true;
			continue;
		}

	}
	fclose(fp1);

	if (!memFound || !swapFound)
		throw procutils_error(
				"[ getSysMem() ] Could not obtain memory info from /proc/meminfo");
	memInfo->totalMem = valMem;
	memInfo->swapMem = valSwap;
#endif
}

string getKernelInfo() throw(procutils_error) {
#ifndef WIN32
	string temp;
	string kernelInfo;
	ifstream fin;

	char szLine[255] = { 0 };
	bool osFound = false;
	try {
		fin.open("/proc/version");
		fin.getline(szLine, 255);

		kernelInfo = szLine;

		std::string::size_type findOffsetStart = 0, findOffsetEnd = 0;
		findOffsetStart = kernelInfo.find_first_of(")");
		kernelInfo = kernelInfo.substr(0, findOffsetStart+1);
		osFound = true;
	} catch (ifstream::failure e) {
		cout << "Exception opening/reading file";
		fin.close();
		throw procutils_error("[ getKernelInfo() ] Could not open /proc/version");
	}
	fin.close();

	if (!osFound)
		throw procutils_error(
				"[ getKernelInfo() ] Could not obtain Kernel info from /proc/version");

	return kernelInfo;
#endif
}

void* sendInformation2(void* data);
int sendInformation() {
  pthread_t thread;
  int data;
  int ret = pthread_create(&thread, 0, sendInformation2, (void*) &data );
  if(ret) {
    ERRLOG("Could not create thread to handle ApMon" << std::endl);
    return -1;
  }
}

void* sendInformation2(void* data) {
	char *destList[2] = {
			(char *) "http://kenobi.kisti.re.kr/amga_monalisa.conf",
			(char *) "kenobi.kisti.re.kr kisti-amga" };

	char logmsg[100];
	int timestamp;

	int parameterLen = 3;
	string parameterName[10] = { "Version", "AMGA Server Type", "IP Address",
			"CPU Frequency", "CPU Vendor", "CPU Model", "# of CPUs",
			"Total Memory", "Swap Memory", "Kernel Info" };
	string parameterValue[10] = { "", };

	try {
                ApMon::setLogLevel((char *) "FATAL");
		ApMon *apm = new ApMon(2, destList);

		string hostName, nodeName;
		char buffer[256] = { 0x00, };
		const char* default_hostName = "Unknown HostDomain";
		const char* localHostName = "localhost";

		// Finding HostName
		int len = 256;
		int ret = gethostname(buffer, len);

		if (ret != 0) {
			strncpy(buffer, default_hostName, strlen(default_hostName));
		}

		if (strstr(buffer, localHostName) != NULL) {
			strncpy(buffer, default_hostName, strlen(default_hostName));
		}

		hostName = buffer;

		// Get IP Address
		int s;
		struct ifreq ifr = { };
		s = socket(PF_INET,SOCK_DGRAM, 0);
		strncpy(ifr.ifr_name, "eth0", sizeof(ifr.ifr_name));
		string ipAddress;
		if (ioctl(s, SIOCGIFADDR, &ifr) >= 0) {
			char *ip = inet_ntoa(
					((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr);
			ipAddress = ip;
		}

		// Get CPU Info
		CPUINFO cpuInfo;

		try {
			getCPUInfo(&cpuInfo);
			std::ostringstream osstream;
			osstream << getNumCPUs();
			cpuInfo.cpuNums = osstream.str();
		} catch (procutils_error &perr) {
			/* "permanent" error (the parameters could not be obtained) */
		} catch (runtime_error &err) {
			/* temporary error (next time we might be able to get the paramerers) */
		}

		// Get Memory Info

		MEMINFO memInfo;
		try {
			getSysMem(&memInfo);
		} catch (procutils_error &perr) {
			/* "permanent" error (the parameters could not be obtained) */
		} catch (runtime_error &err) {
			/* temporary error (next time we might be able to get the paramerers) */
		}
		string kernelInfo;
		try {
			kernelInfo = getKernelInfo();
		} catch (procutils_error &perr) {
			/* "permanent" error (the parameters could not be obtained) */
		} catch (runtime_error &err) {
			/* temporary error (next time we might be able to get the paramerers) */
		}

		nodeName = buffer;
		nodeName += " - " + ipAddress;

#if 0
		sprintf(logmsg, "NodeName : %s \n", nodeName.c_str());
		sprintf(logmsg, "HostName : %s \n", hostName.c_str());
		sprintf(logmsg, "VERSION : %s \n", VERSION);
		sprintf(logmsg, "IP Address : %s \n", ipAddress.c_str());
		sprintf(logmsg, "CPU Frequency: %s \n", cpuInfo.frequency.c_str());
		sprintf(logmsg, "CPU Vendor : %s \n", cpuInfo.vendor.c_str());
		sprintf(logmsg, "CPU Model Name : %s \n", cpuInfo.modelName.c_str());
		sprintf(logmsg, "# of CPUs : %s \n", cpuInfo.cpuNums.c_str());
		sprintf(logmsg, "Toatal Memory : %s \n", memInfo.totalMem.c_str());
		sprintf(logmsg, "Swap Memory : %s \n", memInfo.swapMem.c_str());
		sprintf(logmsg, "Kernel Info : %s \n", kernelInfo.c_str());
#endif 
		/* use the wrapper function with simplified syntax */
		try {
			/* send a datagram with timestamp */
			timestamp = time(NULL);

			parameterValue[0] = VERSION;
			parameterValue[1] = "amgad";
			parameterValue[2] = ipAddress;
			/*
			parameterValue[3] = cpuInfo.frequency + " MHz";
			parameterValue[4] = cpuInfo.vendor;
			parameterValue[5] = cpuInfo.modelName;
			parameterValue[6] = cpuInfo.cpuNums;
			parameterValue[7] = memInfo.totalMem + " KB";
			parameterValue[8] = memInfo.swapMem + " KB";
			parameterValue[9] = kernelInfo;
			*/
			for (int i = 0; i < parameterLen; i++) {
				apm -> sendTimedParameter((char *) "AMGA_SERVER",
						(char *) nodeName.c_str(),
						(char *) parameterName[i].c_str(), XDR_STRING,
						(char *) parameterValue[i].c_str(), timestamp);
			}

		} catch (runtime_error &e) {
		}

		delete apm;
	} catch (runtime_error &e) {
	}
	return 0;
}

/**
 * Main program
 *
 */
int main(int argc, char *argv[]) {
#if __GNUC__ >= 3
	std::set_terminate(__gnu_cxx::__verbose_terminate_handler);
#endif

	int optionIndex = 0;

	bool debug = false;
	bool daemon = false;
	int port = -1 ;
	std::string logFile = "";
	std::string configLocation = "";
	std::string runningMode = "";

	while (1) {
		int c = getopt_long(argc, argv, "hp:dc:l:D", options, &optionIndex);

		if (c == -1)
			break;

		switch (c) {
		case 'h':
			printUsage();
			exit(0);
		case 'd':
			debug = true;
			break;
		case 'D':
			daemon = true;
			break;
		case 'p':
			port = atoi(optarg);
			break;
		case 'l':
			logFile = optarg;
			break;
		case 'c':
			configLocation = optarg;
			break;
		default:
			std::cerr << "Unknown option" << (char) c << std::endl;
			printUsage();
			exit(5);
		}
	}

	if (optind != argc) {
		std::cerr << "Too many parameters!" << std::endl;
		printUsage();
		exit(5);
	}

	ConfigParser::configParser = new ConfigParser();

	if (configLocation != "") {
		// User provided an explicit configuration file.
		if (!ConfigParser::configParser->init(configLocation)) {
			ERRLOG("Could not read configuration: " << configLocation << std::endl);
			exit(5);
		}
	} else {
		// Load the default configuration file
		try {
			configLocation = ConfigParser::loadFromFile("amgad.config",
					*ConfigParser::configParser);
		} catch (runtime_error &ex) {
			ERRLOG("Configuration file not found on the default search paths." << std::endl);
			exit(5);
		}
	}

	// KIMNG090323 Check the mode of AMGA - Process Mode or Thread Mode
	try {
		runningMode = ConfigParser::configParser->getString("Mode");

		if (!((runningMode.compare("process") == 0) || (runningMode.compare(
				"thread") == 0))) {
			runningMode = "process";
		}
		//LOG("Running Mode : " << runningMode << std::endl);
	} catch (std::string err) {
		runningMode = "process";
	}

	// KIMNG090323 Check the mode of AMGA - Process Mode or Thread Mode
	try {
		string report = ConfigParser::configParser->getString("Report");

		if (report.compare("disable") == 0) {
			//LOG("Report : " << report << std::endl);
		}
		else {
			// KIMNG090712 ApMon (Monalisa Client codes)
			sendInformation();
			//LOG("Report : " << report << std::endl);
		}
	} catch (std::string err) {
		sendInformation();
		//LOG("Report : enable " << std::endl);
	}

	AMGAFrame *amga;

	try {
		if (runningMode.compare("process") == 0) {
			// create process instance
			amga = new ProcessFrame();

		} else { // create thread instance
			amga = new ThreadFrame();
		}

		amga->init(*(ConfigParser::configParser), runningMode, debug, daemon,
				port, logFile);
		amga->run();

	} catch (std::exception &ex) {
		ERRLOG("Fatal error: " << ex.what() << endl);
	} catch (...) {
		ERRLOG("Unknown fatal error.");
	}
}

