/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
// $Id: BDIIClient.h,v 1.7 2010/04/14 07:15:30 asunil Exp $
/** BDIIClient is a class which provides the necessary functionality
    to contact a BDII service via LDAP.
**/

#ifndef BDIICLIENT_H
#define BDIICLIENT_H

// Needed for forward compatibility
#define LDAP_DEPRECATED 1

#include <ldap.h>
#include <string>
#include <list>

#include "AMGAService.h"

#define BDII_DEFAULT_PORT 2170

class BDIIClient {
 public:

  class LDAPException {
  public:
    LDAPException(const std::string &message, int e=0);
    std::string getError() const;
    int getErrNum() const { return err; };
  private:
    std::string errString;
    int err;
  };

  BDIIClient(const std::string &host="", int port=0, bool dbg=false)
    throw(LDAPException);

  ~BDIIClient();

  int reset(const std::string &host, int port)
    throw(LDAPException);

  int search(std::list< AMGA::Service > &services,
	     const std::string &base,
	     const std::string &filter)
    throw(LDAPException);

  void setDebug(bool d = true) { debug = d; };

 private:
  LDAP *ldap;
  bool debug;
  std::string who;    // DN of contacting person
  std::string cred;   // Password of contacting person
};

#endif // BDIICLIENT_H
