/*
  Copyright (c) Members of the EGEE Collaboration. 2004.
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/


/** MDFed is a class which provides federation functionality 
**/

#ifndef MDFED_H
#define MDFED_H

#include "MDClient.h"
#include "server/SiteManager.h"

#define ALLOW_FED_EXEC 1024
#define ALLOW_FED_CONN 2048

#define FED_NONE   0
#define FED_ALL    1
#define FED_RANDOM 2
#define FED_FILTER 3
#define FED_ONCE   4
#define FED_SELE   8

#define PROXY_CODE 1000

class MDClient;


class MDFed {

public:
  MDFed(bool dbg=false);
  ~MDFed();
  void setParentMDClient(MDClient* mdc);
  int retrieveFedHosts(std::string& errmsg);
  int initNextFedHosts(int flags , std::string& errmsg);
  void disconnectFed(bool);
  int fetchRow(std::string &line, bool readAhead);
  int eot();
  int initFed(std::string &err, bool server);
  int addFedHost(std::string host, int port, Site& s);
  int decodeFedFlags(std::string& flags);
  int decodeFedFlags(int flags);
  void clearFedHosts();

  // federation
  int cmdType ;
  int allowFedConnFail ;
  int allowFedExecFail ;
  std::string lastCommand;
  bool proxyState;
  std::string configF;
  std::list< AMGA::MDChild >::iterator currentChild;

  int failureCode ;


private:
  MDClient* pmdc;
  bool debug;

  std::list< AMGA::Service > services;
  std::list< AMGA::Service >::iterator currentService;
  std::list< AMGA::MDChild > mdchild;

  int removeFedHeader(int col);
  int lookupFedHosts(std::string &dn) ;


};


#endif
