/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// 
// File ConfigFileLine.C
//
/*! \class ConfigFileLine
 * \brief A class representing a (continued) line in the configuration
 * file, that is a pair of an option name and a value) 
 *  
 *
 * $Date: 2010/04/14 07:15:30 $
 * $Author: asunil $
 *
 * The class ConfigFileLine is used by the config_parser during
 * parsing of the configuration file. Every parsed line is put
 * into a ConfigFile line object. Lines may be continued by the \
 * line continuation character. The name of the configuration option
 * is stored as a string in the "option" class member, while "value"
 * contains a string of a configuration option. The latter field
 * might also be empty if the option is boolean. As soon as the name
 * field contains a non-zero string, the respective option is defined.
 * 
 * The class is used during the parsing proceedure itself and must
 * therefore also be able to contain the data during the parsing
 * step. This includes ConfigFileLine objects which have no option
 * name because they are on the right hand side of the '=' assignment
 * operator.
*/

#include "ConfigFileLine.h"
#include <iostream>

using namespace std;

ConfigFileLine::ConfigFileLine(const string &o, const string &v)
{
  option=o;
  value = v;
}


ConfigFileLine::~ConfigFileLine()
{
  // The "option" string and the list are "self deleting"
  // we do not need to do anything therefore, here.
}


void ConfigFileLine::setValue(const ConfigFileLine &other){
  value = other.getOptionName();
}


void ConfigFileLine::setScope(const string &scope)
{
  if( !scope.size() )
    return;

  string s=scope;
  s.append("::");
  s.append(option);
  option=s;
}


void ConfigFileLine::print() const
{
  std::cout << "Option '" << option <<"' ";
  if( value.size() ) 
    std::cout << "set to '" << value << "'\n";
  else
    std::cout << "is defined!\n";
}
