/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// ConfigFileLine.h
//
// A class representing a (continued) line in the configuration
// file, that is a pair of an option name and a (list of) value(s) 
//
// last change : $Date: 2010/04/14 07:15:30 $
// by          : $Author: asunil $
//

#ifndef CONFIGFILELINE_H
#define CONFIGFILELINE_H

#include <string>
#include <list>

class ConfigFileLine {
 public:

  //! Constructor with name of the option and a single option value (no list!)
  ConfigFileLine(const std::string &option, const std::string &value="");

  //! Destructor: Does currently nothing
  virtual ~ConfigFileLine();

  //! Append the option name of another ConfigFileLine as value
  void setValue(const ConfigFileLine &other);

  //! Check whether name of option is "name"
  bool isOption(const std::string &name) const{
    return name==option;
  }

  //! Check whether the option line has a name, i.e. is defined
  bool isDefined() const{
    return option.length();
  }

  //! Return the list of configuration values
  std::string getValue() const{
    // This operation is somewhat time-consuming!
    return value;
  }

  //! Sets the list of configuration values
  void setValue(const std::string &v){
    value = v;
  }

  //! Get option name
  const std::string &getOptionName() const{
    return option;
  }

  //! Sets the scope by prepending "scope":: to the option name
  void setScope(const std::string &scope);
  
  //! Print the contents of the rule to stdout
  void print() const;

 private:

  std::string option;               //!< The name of the option
  std::string value;                  //!< The option value
};

#endif // #ifndef CONFIGFILELINE
