/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "ConfigParser.h"

#include <iostream>

using namespace std;

int main (int argc, char *argv[])
{
  ConfigParser::ConfigParser configParser;
  if(!configParser.init("example.conf")){
    cout << "Could not read configuration file example.conf\n";
    return 5;
  }
  
  configParser.print();

  try{
    int i=configParser.getInt("useSSL");
    std::cout << "useSSL: " << i << std::endl;
  } catch (std::string e) {
    cout << "Error: " << e << std::endl;
    return 5;
  }

  try{
    double d = configParser.getDouble("version");
    std::cout << "Double version is " << d << std::endl;
  } catch (std::string e) {
    cout << "Error: " << e << std::endl;
    return 5;
  }

  try{
    std::vector< std::string > voList
      = configParser.getStringList("AMGA::VirtualOrganizations");
    
    std::cout << "VirtualOrganizations set to list: '";
    for(size_t i = 0; i< voList.size(); i++ ) {
      std::cout << voList[i];
      if (i != voList.size()-1 )
	std::cout << "', '";
    }
    std::cout << "'\n";

  } catch (std::string e) {
    cout << "Error: " << e << std::endl;
    return 5;
  }
  
  return 0;
}
