/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef DEBUG_H
#define DEBUG_H

#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <sstream>
#include <errno.h>
#include "util/Display.h"

//Macro for <file>, line <line>:
#define __HERE__  __FILE__ ":" QUOTE2(__LINE__)
#define QUOTE(arg) #arg
#define QUOTE2(arg) QUOTE(arg)

#ifdef AMGA_DEBUG

// Thread safe, no garbled output.
#define DMESG(stream) if(debug) { \
	std::ostringstream os; \
	std::string here(__HERE__); \
	size_t start = here.rfind("/"); \
	if(start != std::string::npos) here = here.substr(start+1); \
	os << here << "(" << (void *)pthread_self() << std::dec << ", " \
	<< getpid() << ")"	 << ": " << stream;		     \
	Display::out(os.str()); \
  }

// May produce garbled output if used by several threads.
#define DMESG_UNSAFE(stream) if(debug) { \
	std::string here(__HERE__); \
    size_t start = here.rfind("/"); \
    if(start != std::string::npos) here = here.substr(start+1); \
	logstream << \
	  here << "(" << (int)pthread_self() << std::dec << ", " \
		  << getpid() << ")"	 << ": " << stream; \
  }

#define DMESG_T(stream) if(debug) { \
  char timebuf[50]; \
  time_t timet; \
  time(&timet); ctime_r(&timet, timebuf); \
  std::string here(__HERE__);		  \
  size_t start = here.rfind("/"); \
  if(start != std::string::npos) here = here.substr(start+1); \
  logstream << \
    __HERE__ << "( at " << timebuf << ")" \
	    << ": " << stream;		  \
  }
#else

#define DMESG(stream)
#define DMESG_T(stream)
#define DMESG_UNSAFE(stream)

#endif /* AMGA_DEBUG */

#define LOG(stream) { \
  char timebuf[50]; \
  time_t timet; \
  time(&timet); ctime_r(&timet, timebuf); \
  if (strlen(timebuf)) timebuf[strlen(timebuf)-1] = ' ';  \
  std::ostringstream os; \
  os << timebuf << " " << stream; \
  Display::out(os.str()); \
  }

#define LOG_UNSAFE(stream) logstream << stream
#define ERRLOG_UNSAFE(stream) logstream << stream
#define ERRLOG(stream) LOG(stream)
#define PERRLOG_UNSAFE(stream) if(1) {		         \
    char logbuff[201]; logbuff[0]='\0';                  \
    strcpy(logbuff, strerror_r(errno, logbuff, 200));	 \
    logstream << stream << ": " << logbuff << std::endl; \
    \
  }

#define PERRLOG(stream) if(1) {		      \
    char logbuff[201]; logbuff[0]='\0';       \
    strcpy(logbuff, strerror_r(errno, logbuff, 200));	 \
    LOG(stream << ": " << logbuff << std::endl);	 \
    \
  }

#endif /* DEBUG_H */

