/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "Function.h"

namespace QueryParser{

Function::Function(const int size, const std::string &name)
{
  function=name;
  args.resize(size);
  for(size_t i=0; i<args.size(); i++)
    args[i]=0;
}

Function::~Function()
{
  for(size_t i=0; i<args.size(); i++)
    delete args[i];
}


int Function::toSQL(const std::string &prefix, std::string &query,
		    const std::string &style)
{
  std::string fn(function);

  if( fn == "log" && style.find("MySQL")!= std::string::npos )
    fn = "log10";

  if( fn == "log" && style.find("Oracle") != std::string::npos ) {
    query.append("log(10, ");
    if( args.size() )
      args[0]->toSQL(prefix, query, style);
    query.append(") ");
    return function.size();
  }
  
  if( fn == "pow" && style.find("Oracle") != std::string::npos )
    fn = "power";

  if( fn == "rnd" ) {
    if( style.find("MySQL") != std::string::npos )
      fn = "rand";
    else
      fn = "random";
  }

  if(fn == "like") {
    query.append("(");
    if( args.size() )
      args[0]->toSQL(prefix, query, style);
    query.append(" LIKE ");
    if( args.size()>1 )
      args[1]->toSQL(prefix, query, style);
    query.append(")");
    return function.size();
  }

  if(fn == "if") {
    query.append("( CASE WHEN");
    if( args.size() )
      args[0]->toSQL(prefix, query, style);
    query.append(" THEN ");
    if( args.size()>1 )
      args[1]->toSQL(prefix, query, style);
    if( args.size()>2 ) {
      query.append(" ELSE ");
      args[2]->toSQL(prefix, query, style);
    }
    query.append("END)");
    return function.size();
  }

  if(fn == "isnull") {
    query.append("(");
    if( args.size() )
      args[0]->toSQL(prefix, query, style);
    query.append(" IS NULL");
    query.append(")");
    return function.size();
  }

  if(fn == "notnull") {
    query.append("(");
    if( args.size() )
      args[0]->toSQL(prefix, query, style);
    query.append(" IS NOT NULL");
    query.append(")");
    return function.size();
  }
		 
  query.append(fn);
  query.append("(");
  
  if(args.size()) {
    args[0]->toSQL(prefix, query, style);
    for(size_t i=1; i<args.size(); i++){
      query.append(", ");
      args[i]->toSQL(prefix, query, style);
    }
  }
  query.append(") ");

  return function.size();
}


void Function::addToTableMap(TableMap &tableMap)
{
  for(size_t i=0; i<args.size(); i++)
    args[i]->addToTableMap(tableMap);
}


void Function::replaceTableNames(TableMap &tableMap)
{
  for(size_t i=0; i<args.size(); i++)
    args[i]->replaceTableNames(tableMap);
}

}
