/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "Join.h"
#include <stdlib.h>
#include <stdio.h>


namespace QueryParser{

Join::Join(int type, Expression *l, Expression *r, Expression *c)
  : joinType(type), left(l), right(r), on(c)
{

}


Join::~Join()
{
  delete left;
  delete right;
  if( on )
    delete on;
}


int Join::toSQL(const std::string &prefix, std::string &query,
		 const std::string &style)
{
  switch (joinType) {
  case JOIN_CROSS:
    query += " (";
    left->toSQL(prefix, query, style);
    query += " CROSS JOIN ";
    right->toSQL(prefix, query, style);
    if( on )
      on->toSQL(prefix, query, style);
    query += ")";
    if( on ) {
      query += " ON ";
      on->toSQL(prefix, query, style);
    }
    break;
  case JOIN_LEFT_USING:
  case JOIN_LEFT_ON:
    query += " (";
    left->toSQL(prefix, query, style);
    query += " LEFT JOIN ";
    right->toSQL(prefix, query, style);
    if( on ) {
      if( joinType == JOIN_LEFT_USING)
	query += " USING(";
      else
	query += " ON ";
      on->toSQL(prefix, query, style);
      if( joinType == JOIN_LEFT_USING)
	query += ")";
    }
    query += ")";
    break;
  case JOIN_RIGHT_USING:
  case JOIN_RIGHT_ON:
    query += " (";
    left->toSQL(prefix, query, style);
    query += " RIGHT JOIN ";
    right->toSQL(prefix, query, style);
    if( on ) {
      if( joinType == JOIN_RIGHT_USING)
	query += " USING(";
      else
	query += " ON ";
      on->toSQL(prefix, query, style);
      if( joinType == JOIN_RIGHT_USING)
	query += ")";
    }
    query += ")";
    break;
  case JOIN_INNER_USING:
  case JOIN_INNER_ON:
    query += " (";
    left->toSQL(prefix, query, style);
    query += " INNER JOIN ";
    right->toSQL(prefix, query, style);
    if( on ) {
      if( joinType == JOIN_INNER_USING)
	query += " USING(";
      else
	query += " ON ";
      on->toSQL(prefix, query, style);
      if( joinType == JOIN_INNER_USING)
	query += ")";
    }
    query += ")";
    break;
  }
  return 1;
}

void Join::addToTableMap(TableMap &tableMap)
{
  left->addToTableMap(tableMap);
  right->addToTableMap(tableMap);
  if( on )
    on->addToTableMap(tableMap);
}


void Join::replaceTableNames(TableMap &tableMap)
{
  left->replaceTableNames(tableMap);
  right->replaceTableNames(tableMap);
  if( on )
    on->replaceTableNames(tableMap);

}

}
