/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/** QParser is a class which parses legal queries for the metadata
    server, currently the following syntax is supported:
    query: query expression
    expression: attribute | constant | (expression) | function | token
                          | expression op expression
    op: = | < |  > | <= | >= | != | <> | + | - | * | / | and | or
    function: fname (expression)
    token: string | number | attribute
    
    Valid functions are currently: lower

    Attributes need to start with a letter. Numbers (which include
    floats) with a digit.
    Attributes can be qualified by a path of a collection: /path:attr
    and may refer to ranges of an array: attr[2], or attr[2:5].
    Strings are enclosed by double quotes ("").

    Valid queries can be empty strings.

    In the protocol queries containing spaces are enclosed by single quotes.
**/

#ifndef QPARSER_H
#define QPARSER_H

#include <string>
#include "ScannerContext.h"
#include "util/TableEntry.h"

class QParser {
 public:
  QParser(const std::string &prefix, bool debug=false);
  ~QParser();

  void setEngine(const std::string &e) { engine = e; };

  int parse(const std::string &query, std::string &result, bool plain = false);
  int parse(const std::string &query);
  int toSQL(std::string &result, bool plain = false);

  const std::string &prefix() const { return pref; };

  void addToTableMap(TableMap &tableMap);
  void replaceTableNames(TableMap &tableMap);

  int getConstraint(std::string &result, bool plain=false);
  
  int getJoin(std::string &result, bool plain=false);
    
  bool distinctOnly() const;

 private:
  std::string pref;
  ScannerContext scanner;
  std::string strbuf;
  std::string engine;
  bool debug;
};

#endif
