/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "Relation.h"
namespace QueryParser{

Relation::Relation(const std::string &op)
  : left(NULL), right(NULL)
{
  relation=op;
}

Relation::~Relation()
{
  delete left;
  delete right;
}


int Relation::toSQL(const std::string &prefix, std::string &query, const std::string &style)
{
  if(left){
    if(left->isCompound())
      query.append(" (");
    left->toSQL(prefix, query, style);
     if(left->isCompound())
      query.append(") ");
  }

  query.append(" ").append(relation).append(" ");

  if(right){
    if(right->isCompound())
      query.append(" (");
    right->toSQL(prefix, query, style);
    if(right->isCompound())
      query.append(") ");  

  }

  return relation.size();
}



void Relation::addToTableMap(TableMap &tableMap)
{
  if(left)
    left->addToTableMap(tableMap);
  if(right)
    right->addToTableMap(tableMap);
}


void Relation::replaceTableNames(TableMap &tableMap)
{
  if(left)
    left->replaceTableNames(tableMap);
  if(right)
    right->replaceTableNames(tableMap);
}
}
