/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


//
// $Id: TableColumn.cc,v 1.5 2010/04/14 07:15:30 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "TableColumn.h"
#include "Table.h"

namespace QueryParser{

TableColumn::TableColumn(Identifier *ident, Table *t)
  : table(t), id(ident)
{
 
}


TableColumn::TableColumn(Identifier *ident)
  : id(ident)
{
  table = NULL;
}

TableColumn::~TableColumn()
{
  if( table ) delete table;
  delete id;
}


int TableColumn::toSQL(const std::string &prefix, std::string &query,
			   const std::string &style)
{
  table->toSQL(prefix, query, style);
  query.append(".");
  int res;
  if( table->getOptions() & EP_PLAIN )
    res = id->toSQL("", query, style);
  else
    id->toSQL(prefix, query, style);
  return res;
}


void TableColumn::addToTableMap(TableMap &tableMap)
{
  if( table )
    table->addToTableMap(tableMap);
}


void TableColumn::replaceTableNames(TableMap &tableMap)
{
  if( table )
  table->replaceTableNames(tableMap);
}


int TableColumn::getOptions() const
{ 
  return table->getOptions(); 
}


}
