/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef COMMANDPARSER_H_
#define COMMANDPARSER_H_

#include "util/Socket.h"
#include "util/Debuggable.h"

#include <vector>
#include <map>
#include <ostream>
#include <exception>

#define MAX_COMMAND_LENGHT 128

/**
 * Commands recognized by the mdrepdaemon
 */
struct Command {
    enum CommandID {
        cSubscribe, 
        cUnsubscribe, 
        cSubscribeUsers, 
        cUnsubscribeUsers,
        cUnsubscribeAll,
        cSynchronize, 
        cQuit,
        cReceive 
    };

    Command::CommandID cId;
    std::map<std::string, std::string> optionalArgs; 
    std::vector<std::string> args;
    
    friend std::ostream& operator<<(std::ostream& out, const Command &cmd);
};


class CommandParseException : public std::exception {
public:
  CommandParseException(const std::string &message) throw ()
  : userMessage(message)  {}
  
  virtual ~CommandParseException() throw () {};
  
  const char *what() const throw() {
    return userMessage.c_str();
  };
  
private:
  // Exception message
  std::string userMessage;  
};


class CommandParser : protected Debuggable
{
public:
	CommandParser(TCPSocket &socket);
	virtual ~CommandParser();
    
    void nextCommand(Command &cmd) throw (SocketException, CommandParseException);
    void skipToEndOfCommand() throw (SocketException);

private:
    TCPSocket& _socket;
    
//    static std::string SUBS_INHERITH = "inherit";
//    static std::string SUBS_XID = "xid";

    /** 
     * Maps between the string representing a command (sent by the 
     * client) and the numerical constant used internally
     */
    std::map<std::string, Command::CommandID> _commands;
};

#endif /*COMMANDPARSER_H_*/
