/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef MDLOGWRITER_H_
#define MDLOGWRITER_H_

#include "util/DBUtil.h"

#include <string>
#include <stdint.h>

class Statement;

const std::string USER_CMD_META_DIR = "<users>";

class MDLogWriter {
private:
	std::string _nextCmd;
  uint64_t _xid;
  bool _inTransaction;
	static bool _saveLogs;
  static bool debug;
  
public:
	MDLogWriter();
	virtual ~MDLogWriter();
  
	void setNextCommand(const std::string& nextCmd);

    /** Saves a replication log with the metadata command saved by 
     * the setNextCommand method. 
     */
	void saveStoredLog(Statement &statement, 
        const std::string &absDirectory, // The directory updated
        const std::string &user, // The user that executed the command
        const std::string &currentDir, // The context directory
        const std::string &permissionMask, // The user that executed the command
        const std::string &groupRightsMask) throw (SQLException);

  /** Saves a replication log with the given metadata command.
   */
  void saveLogCommand(Statement &statement, 
      const std::string &command, // The command to be saved
      const std::string &absDirectory, // The directory updated
      const std::string &user, // The user that executed the command
      const std::string &currentDir, // The context directory
      const std::string &permissionMask, // The user that executed the command
      const std::string &groupRightsMask) throw (SQLException);

	static void setSaveLogs(bool saveLogs);	
	static bool getSaveLogs();
  
  static void setDebug(bool debug); 
  static bool getDebug();
  
  void beginTransaction(
      Statement &statement,
      const std::string &user, // The user that executed the command
      const std::string &permissionMask,
      const std::string &groupRightsMask) throw (SQLException);
      
  void beginUpload(Statement &statement,
      const std::string &absDirectory, // The directory updated
      const std::string &user, // The user that executed the command
      const std::string &cwd, // The directory updated
      const std::string &permissionMask,
      const std::string &groupRightsMask) throw (SQLException);
      
  void commit();
  void abort();
  
private:
    bool isSubscribed(Statement &statement, const std::string &absDirectory) 
      throw (SQLException);    
      
    bool transactionInProgress();
    uint64_t getLastLogXid(Statement &st) const 
      throw (SQLException);
      
   uint64_t lastXid;
   
   void setupTableNames();

   std::string transactionsTable;
   std::string logsTable;
   std::string subscriptionsTable;
   std::string subscribersTable;
};

#endif /*MDLOGWRITER_H_*/
