/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef REPLICATIONMASTER_H_
#define REPLICATIONMASTER_H_

#include "util/Debuggable.h"
#include "util/DBUtil.h"
#include "util/StatsCollector.h"
#include "server/Constants.h"

#include <string>
#include <map>
#include <stdexcept>

class ConfigParser;

/**
 * In the future this class might be extended to manage everything
 * related to replication, including creating the necessary objects
 * and holding the configuration. Currently it is just a simple
 * prototype that holds the name of the node.
 */
class ReplicationManager : public Debuggable, StatsSource
{
public:
	virtual ~ReplicationManager();

  static void initManager(const ConfigParser &configParser, bool debug, bool asRepDaemon = false) 
    throw (std::runtime_error, SQLException);

  static void destroy();
  static ReplicationManager* getInstance() {
      return instance;
  };

  /** Returns the name of the this node */
  std::string getNodeName() const {
      return nodeName;
  };
  bool isMaster() const { return enableMaster; };
  bool isSlave() const { return enableSlave; };
  int getMaxPendingLogs() const { return maxPendingLogs; };
      
  virtual void collectCounters(std::map<std::string, std::string> &counters);
  
private:
    static ReplicationManager *instance;
    ReplicationManager(const ConfigParser &configParser, bool debugInit, bool asRepDaemon = false)
     throw (std::runtime_error, SQLException);

    void initAsSlave(const ConfigParser &configParser);
    void initAsMaster(const ConfigParser &configParser);
    void initAsRepDaemon(const ConfigParser &configParser);
    
    std::string nodeName;
    /** Enables connecting to other nodes as a slave */
    bool enableSlave;
    /** Enables saving replication logs */
    bool enableMaster;
    /** Enables saving replication logs */
    bool enableRepDaemon;    
    /** Maximum size of logs table before dropping subscribers */
    int maxPendingLogs;
    
};

#endif /*REPLICATIONMASTER_H_*/
