/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: DBUserManager.h,v 1.7 2010/04/14 07:15:30 asunil Exp $
//
/** DBUserManager is a class to manage users, and their credentials
    like a (hashed) password or an X509 certificates subject name.
    While a user can have only one password authenticating him,
    there is the possibility to have several certificates
    authenticating a single user. However, it is not possible for the
    same certificate to be used to authenticate for several users.
    If you need this, you will need to use the group features of
    MDServer.
**/

#ifndef DB_USER_MANAGER_H
#define DB_USER_MANAGER_H

#include "MDUserManager.h"

class DatabaseConnection;

class DBUserManager: public MDUserManager {
 public:
  DBUserManager();
  DBUserManager(DatabaseConnection *dbConn);
  ~DBUserManager();

  void setConnection(DatabaseConnection *dbConn);

  int connectDataSource(const std::string &dataSource,
			const std::string &dbUser,
			const std::string &dbPass);

  virtual bool checkSubject(const std::string &user,
			    const std::string &subject) const;

  virtual  bool checkPass(const std::string &user,
			  const std::string &pass) const;

  virtual bool isKnownUser(const std::string &user) const;
  virtual bool isKnownSubject(const std::string &subject) const;

  virtual int addUser(const std::string &user,
		      const std::string &pass="", bool passIsHash = false);

  virtual int addUserSubject(const std::string &user,
			     const std::string &subject);

  virtual int removeUserSubject(const std::string &user,
				const std::string &subject);

  
  virtual int changeUserPass(const std::string &user,
			     const std::string &pass);

  virtual int removeUser(const std::string &user);

  virtual int listUserCred(const std::string &user, UserCred &cred);

  virtual int listUsers(std::list < std::string > &users);

 private:
  int disconnect();
  
  void setupTableNames();

  bool ownsConnection;
  DatabaseConnection *dbConn;

  bool initialized;

  std::string usersTable;
  std::string certificatesTable;
};

#endif //DB_USER_MANAGER_H
