/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: GridMapUserManager.h,v 1.4 2010/04/14 07:15:30 asunil Exp $
//
/** GridMapUserManager is a class to which can read a grid map file
    which maps certificate subjects to a user name.
    This user information is read-only.
**/

#ifndef GRID_MAP_USER_MANAGER_H
#define GRID_MAP_USER_MANAGER_H

#include "MDUserManager.h"
#include <map>

class GridMapUserManager: public MDUserManager {
 public:
  GridMapUserManager();
  virtual ~GridMapUserManager();

  int initFromGridMap(const std::string &file);

  virtual bool checkSubject(const std::string &user,
		    const std::string &subject) const;

  virtual bool checkPass(const std::string &user,
		 const std::string &pass) const;

  virtual bool isKnownSubject(const std::string &subject) const;

  virtual int listUserCred(const std::string &user, UserCred &cred);

  virtual int listUsers(std::list< std::string > &users);

 private:
  int parseLine(const std::string &line,
		std::string &subject, 
		std::string &role);

  std::map< std::string, std::string > certificates;
};

#endif //GRID_MAP_USER_MANAGER_H
