/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/** MDBuffer is a class which implements an asynchronous buffer which
    sits in between MDServer and the server program
**/

#ifndef MDBUFFER_H
#define MDBUFFER_H

#include <string>

class MDBuffer {
 public:
  MDBuffer();
  virtual ~MDBuffer();
  
  virtual void start();
  virtual void cancel();
  virtual void append(const std::string &text);
  virtual void append(int value);

  virtual bool isProducerFinished() const;
  virtual bool isCancelled() const;  
  virtual bool isAsynchronous() const;

  virtual void read(std::string &buf, int timeout = -1);

  virtual void flush(std::string &result);
  virtual void flush() {};

  size_t getSize() const { return buffer.size(); };

  void print(std::string &buf) { buf = buffer; };

 protected:

  std::string buffer;
  bool producerFinished;
  bool cancelled;
};

#endif
