/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/** -*- Mode:c++ -*-
    $Id: MDConManDB.h,v 1.4 2010/04/14 07:15:30 asunil Exp $

    MDConManDB extends MDConMan by a session cache implemented via
    the GDBM database.
**/

#ifndef MDCONMANDB_H
#define MDCONMANDB_H

#include <gdbm.h>
#include "MDConMan.h"

class MDConManDB: public MDConMan {
 public:
  MDConManDB();
  MDConManDB(int nprocs);
  virtual ~MDConManDB();

  virtual unsigned char *saveSessionSSL(const char *data, int size, 
					unsigned char *sID,
					unsigned int sIDLen);

  virtual uint64_t saveSessionUser(const UserContext &context,
				   uint64_t sID = 0);

  virtual int getSessionSSL(unsigned char *sID, unsigned int sIDLen,
			    char *data, int &size);

  virtual int getSessionUser(const uint64_t &sID, UserContext &context);

  virtual  int deleteSession(unsigned char *sID, unsigned int sIDLen);

  virtual int initSessionCache(int sessions=1024, int maxSize=3000,
			       const std::string &file="");

  int closeSessionCache();
  int expireSessions(int age = 1000);

  //Statistics
  virtual int getUsedSessions() const;
  virtual std::string getSessionStorage() const;

private:
  static std::string dbFile;
  GDBM_FILE db;
};

#endif
