/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/** -*- Mode:c++ -*-
    $Id: MDConManMem.h,v 1.4 2010/04/14 07:15:30 asunil Exp $

    MDConManMem extends the MDConMan connection manager with a session
    cache implemented using shared memory.
**/

#ifndef MDCONMANMEM_H
#define MDCONMANMEM_H

#include "MDConMan.h"

// Key length must be larger than 8!
#define MAX_KEYLEN 64

class MDConManMem: public MDConMan {
 public:
  MDConManMem();
  MDConManMem(int nprocs);
  virtual ~MDConManMem();
  
  unsigned char *saveSessionSSL(const char *data, int size, 
				unsigned char *sID, unsigned int sIDLen);

  uint64_t saveSessionUser(const UserContext &context, uint64_t sID = 0);

  int getSessionSSL(unsigned char *sID, unsigned int sIDLen,
		    char *data, int &size);

  int getSessionUser(const uint64_t &sID, UserContext &context);

  int deleteSession(unsigned char *sID, unsigned int sIDLen);

  int initSessionCache(int sessions=1024, int maxSize=3000,
		       const std::string &file="");

  int closeSessionCache();
  int expireSessions(int age = 1000);


  virtual int getUsedSessions() const;
  virtual std::string getSessionStorage() const;

 private:
  struct Hash {
    short keyLen;
    char key[MAX_KEYLEN];
    short dSize;
  };
  
  int getNewHashSlot(unsigned char *key, unsigned int len);
  int getHashSlot(unsigned char *key, unsigned int len);
  uint32_t hash(unsigned char *key, unsigned int len);
  
  struct Hash *hashMem;
  uint32_t hashBits; // Bits need to be masked out for the hash
  int hashSize; // How many bytes are used for the hash

  int hashFD; // File descriptor of the shared memory file
  static std::string hashMapName;
};

#endif
