/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: MDLFCServer.h,v 1.19 2010/10/26 07:27:47 asunil Exp $
//
#ifndef MDLFCSERVER_H
#define MDLFCSERVER_H

#include "MDServer.h"

class MDLFCServer: public MDServer {
 public:
  MDLFCServer(MDBuffer &outbuf);
  virtual ~MDLFCServer();

  // Help
  virtual void help();

  // Metadata commands
  virtual void createSchema(const std::string &name,
			    AttrList &attributes,
			    const std::string &parent = "");

  virtual void removeSchema(const std::string &name);

  virtual void addEntry(const std::string &paths, 
			std::vector< std::string > &attributes,
			std::vector< std::string > &values, bool eval = false);

  virtual void addEntries(std::vector<std::string> &paths);

  virtual void removeEntries(std::vector<std::string> &paths);

  virtual void getAttr(const std::string &path,
		       std::vector< std::string > &attributes);

  virtual void find(const std::string &path,
		    const std::string &query="");

  virtual void listEntries(const std::string &dir=".");

  // Metadata optional
  virtual void createView(const std::string &name,
			  const std::string &directory,
			  const std::vector<std::string> &attributes,
			  const std::string &restriction);

  virtual void createViewUnion(const std::string &name,
			       const std::vector<std::string> &dirs,
			       const std::string &restriction);

  virtual void createIndex(const std::string &name, const std::string &dir,
			   const std::vector< std::string > &attributes,
			   const std::string &algo="BTREE");

  virtual void createSequence(const std::string &name,
			      const std::string &dir,
			      const std::string &increment,
			      const std::string &start);

  virtual void indexRemove(const std::string &path);

  virtual void viewRemove(const std::string &path);

  virtual void selectAttr(std::vector<std::string> &attributes,
			  const std::string &condition);

 int upload(UploadHandle &handle,
  			  const std::string &dir,
		      std::vector< std::string > &attributes);

  int put(UploadHandle &h,
		      const std::vector<std::string>::iterator valStart,
		      const std::vector<std::string>::iterator valEnd);		        	

  void abort(UploadHandle &handle);
  void commit(UploadHandle &handle);  

  virtual void sequenceNext(const std::string &name);

  virtual void sequenceCurrent(const std::string &name);

  virtual void sequenceRemove(const std::string &path);

  // File-catalogue support
  virtual void dir(const std::string &dir=".");
  
  void setLFCSchema(const std::string &schema) { lfcSchema = schema; };

  virtual void setupTableNames();

protected:
  virtual int getEntryProps(const std::string &entry,
			    EntryPropList &props,
			    const std::string &indexTable = "masterindex",
			    const bool = true);

  virtual void doBulkRead(Statement &statement, std::string &query,
			  size_t fields, bool &foundSomething) = 0;

  virtual std::string createTable(const EntryProps &p,
				  AttrList &attrs,
				  Statement &statement) = 0;

  std::string lfcSchema;

//  virtual uint64_t sequenceNextValue(const std::string &sequence) 
//    throw (SQLException);
    

private:

  virtual int memberOf(const std::string usr,
		       std::list< std::string > &groups) const;

  std::string permsToString(int filemode, int mode);

};

#endif // MDLFCSERVER_H
