/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: MDOracleServer.h,v 1.10 2010/10/25 05:26:29 asunil Exp $
//
#ifndef MDORACLESERVER_H
#define MDORACLESERVER_H

#include "MDStandalone.h"

class MDOracleServer: public MDStandalone {
 public:
  MDOracleServer(MDBuffer &outbuf);
  virtual ~MDOracleServer();
  virtual void vacuum(const std::string &options);
  
protected:
  std::string createTable(const EntryProps &p,
			  AttrList &attrs, Statement &statement);
  virtual int dumpIndexes(Statement &statement,
                          const std::string &dir,
                          const std::string &table) ;

private:
  virtual bool alterTableColumns(const std::string &tableName,
				 const AttrList &attribtues,
				 bool remove);
  virtual int lockTable(Statement &statement, 
			const std::string &indexTable);

  virtual int getAttributes(const std::string &table,
			    AttrList &keys,
			    bool all=false);

  virtual std::string matchSubdirs(const std::string &dir) const;
  virtual bool dirPattern(const std::string &in, std::string &out) const;
  virtual std::string opOnly() const { return ""; };
  virtual std::string opRegExpMatch() const { return "REGEXP_LIKE"; };

  virtual void doBulkRead(Statement &statement, std::string &query,
			  size_t fields, bool &foundSomething);
  
  virtual bool hasCursors() const { return true; };
  virtual bool hasExists() const { return false; }
};

#endif // MDORACLESERVER_H
