/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "MDSocketBuffer.h"

#include "util/Socket.h"

#include <stdexcept>
#include <iostream>
using namespace std;

MDSocketBuffer::MDSocketBuffer(TCPSocket *socketInit) : socket(socketInit)
{}

MDSocketBuffer::MDSocketBuffer() : socket(NULL)
{}

MDSocketBuffer::~MDSocketBuffer() {
}


bool MDSocketBuffer::isProducerFinished() const
{
    throw logic_error("Operation not supported");
}

void MDSocketBuffer::append(const std::string &text) {
//  cout << "Writing: " << text << endl;  
  char controlBuf;
  if (socket->recvOutOfBand(&controlBuf, 1) > 0) {
    // Client requested cancellation.
    cancelled = true;    
  }
    
  buffer.append(text);
  if (buffer.size() > 4*1024) {
    socket->send(buffer);
    buffer.clear();
  }
}

void MDSocketBuffer::setSocket(TCPSocket *socket) {
  this->socket = socket;
}

TCPSocket *MDSocketBuffer::getSocket() {
  return socket;
}

void MDSocketBuffer::append(int value)
{
  char buff[21];
  snprintf(buff, 20, "%d", value);  
  append(buff);
}

void MDSocketBuffer::read(std::string &buf, int timeout) {
  throw logic_error("Operation not supported");
}

void MDSocketBuffer::flush(std::string &result) {
  throw logic_error("Operation not supported");
}

void MDSocketBuffer::flush() {  
  if (!buffer.empty()) {
    socket->send(buffer);
    buffer.clear();
  }
}
