/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "MDThreadMan.h"

static bool debug = false;

MDThreadMan::MDThreadMan()
{
}

MDThreadMan::~MDThreadMan()
{
	pthread_mutex_destroy(&db_sync_mutex);
	pthread_mutex_destroy(&accept_sync_mutex);
}

// KIMNG040821 initialize member variables
int MDThreadMan::init(int nThread, int port) {

	threadNum = nThread;

	threads = new Thread[threadNum];	// KIMNG080418 create a struct Thread array as the thread number

	for (int i = 0; i < threadNum; i++) {
		  // initialize Thread struct
		  threads[i].sessionID=threads[i].status=threads[i].threadIndex=threads[i].timestamp=0;
		  threads[i].user[0] ='\0';
		  //threads[i].userContext = NULL; // assign the User Conext
	}

	runThreadNum = 0;

	serverPort = port;

	return initLock();
}

void MDThreadMan::setThreadNum(int nThread) {
	threadNum = nThread;
}

void MDThreadMan::setConnection(MDServer *mdServer) {
	connection = mdServer;
}

int MDThreadMan::initLock() 
{

	if (pthread_mutex_init(&db_sync_mutex, NULL) < 0) {
		DMESG("error : Failure of initializing the pthread's mutex variable " << std::endl);
		return -1;
	}

	if (pthread_mutex_init(&accept_sync_mutex, NULL) < 0) {
		DMESG("error : Failure of initializing the pthread's mutex variable " << std::endl);
		return -1;
	}

	DMESG("Succcess of initializing the pthread's mutex variable" << std::endl);

	return 0;
}

void MDThreadMan::setStatus(int threadIndex, int status) {
	//DMESG("Thread [" << threadIndex << "] : Status Change. (" << threads[threadIndex].status << " -> " << status << ")" << std::endl);
	threads[threadIndex].status = status;
}

int MDThreadMan::getStatus(int threadIndex) {
	return threads[threadIndex].status;
}

int MDThreadMan::getDBLock(int threadIndex, int indicate) {
    int ret;
	DMESG("TRY a lock : DATABASE -Thread[" << threadIndex << "][" << indicate << std::endl);
	ret = pthread_mutex_lock(&db_sync_mutex);
	DMESG("GET a lock : DATABASE - Thread[" << threadIndex << "][" << indicate << std::endl);
	return ret;
}

int MDThreadMan::releaseDBLock(int threadIndex, int indicate) {
	DMESG("Release a Lock : DATABASE - Thread[" << threadIndex << "][" << indicate << std::endl);
	return pthread_mutex_unlock(&db_sync_mutex);
}

int MDThreadMan::getAcceptLock(int threadIndex) {
    int ret;
	//DMESG("TRY a lock : ACCEPT - Thread[" << threadIndex << "]" << std::endl);
	ret = pthread_mutex_lock(&accept_sync_mutex);
	//DMESG("GET a lock : ACCEPT - Thread[" << threadIndex << "]" << std::endl);
	return ret;
}

int MDThreadMan::releaseAcceptLock(int threadIndex) {
	//DMESG("Release a Lock : ACCEPT - Thread[" << threadIndex << "]" << std::endl);
	return pthread_mutex_unlock(&accept_sync_mutex);
}

void MDThreadMan::setSock(int threadIndex, auto_ptr<TCPSocket> socket) {
	threads[threadIndex].sock = socket;
}

auto_ptr<TCPSocket> MDThreadMan::getSock(int threadIndex) {
	return threads[threadIndex].sock;
}

void MDThreadMan::setDBConnection(DatabaseConnection *db) throw (SQLException) {
	dbConn = db;
}

DatabaseConnection *MDThreadMan::getDBConnection() throw (SQLException) {
	return dbConn;
}
