/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef MDTHREADMAN_H_
#define MDTHREADMAN_H_

#include <pthread.h>
#include <vector>
#include "UserContext.h"
#include "util/StatsCollector.h"
#include "debug.h"
#include <memory>
#include <string.h>
#include <stdlib.h>
#include "MDStandalone.h"
#include "util/Socket.h"
#include "server/MDSocketBuffer.h"
#include "util/DBUtil.h"

using namespace std;

class DatabaseConnection;

//KIMNG080418 Declartion of the class to share information between threads
class MDThreadMan
{
public:
	MDThreadMan();
	virtual ~MDThreadMan();

private :

	// KIMNG080416 the struct variable that has the information of a thread
	struct Thread {
		int threadIndex;
		time_t timestamp;
		int16_t status;
		uint64_t sessionID;
		char user[USERNAME_MAX_SIZE];
		// UserContext for SSL Callback
		//UserContext userContext;
		auto_ptr<TCPSocket> sock;
	};

	struct Thread *threads; // KIMNG080418 the array of a struct variable for thread information

	// the thread number
	int threadNum;
	
	// the running thread number to respond the clients' request
	int runThreadNum;

	// variables for syncronization betwwen threads
	pthread_mutex_t db_sync_mutex;

	pthread_mutex_t accept_sync_mutex;
	
	int serverPort;
	
	// KIMNG080514 Database Connection instance
	DatabaseConnection *dbConn;

	// KIMNG080423 initilize the condition variable
	int initLock();
	
	TCPServerSocket *serverSock;
	
public :
	
	// DB Connection. This variables should be shared betwwen threads
	MDServer *connection;
	
	// KIMNG040821 initialize member variables
	int init(int threadNum, int port);
	
	void setThreadNum(int threadNum);
	
	void setConnection(MDServer *connection);

	void setStatus(int threadIndex, int status);	
	
	int getStatus(int threadIndex);
	
	//void setBuffer(TCPSocket *socket);
	
	int getServerPort() {return serverPort;}
	
	int getThreadNumbers() { return threadNum;}
	int increaseThreadNumbers() { return ++threadNum;}
	int decreaseThreadNumbers() { return --threadNum;}
	
	int getRunningThreadNumbers() { return runThreadNum;}
	int increaseRunningThreadNumbers() { return ++runThreadNum;}
	int decreaseRunningThreadNumbers() { return --runThreadNum;}

	void setSock(int threadIndex, auto_ptr<TCPSocket> socket);
	auto_ptr<TCPSocket> getSock(int threadIndex);

	int getDBLock(int threadIndex, int indicate = 0);
	int releaseDBLock(int threadIndex, int indicate = 0);
	
	int getAcceptLock(int threadIndex);
	int releaseAcceptLock(int threadIndex);
	
	DatabaseConnection *getDBConnection() throw (SQLException);
	void setDBConnection(DatabaseConnection *db) throw (SQLException);
	
	void setServerSock(TCPServerSocket *server) { serverSock = server;}; 
	TCPServerSocket *getServerSock() { return serverSock;};
}; 
#endif /*MDTHREADMAN_H_*/
