/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "MySQLHelper.h"
#include "debug.h"
#include "util/Statement.h"

static bool debug = false;

std::string MySQLHelper::mdToDBType(const std::string &type) const
{
  std::string t;

  if(type == "int")
    return "integer";

  if(type == "float")
    return "double precision";

  if( type == "timestamp" )
    return "datetime";

  if(type.substr(0, 7) == "varchar") {
    t = "character varying";
    t.append(type.substr(7));
    return t;
  }

  return type;
}
 

std::string MySQLHelper::dbToMDType(const std::string &type) const
{
  std::string t;

  if (type == "double precision")
    return "float";

  if (type == "double" || type.substr(0, 6) == "bigint")
    return "float";

  if (type.substr(0, 3) == "int" || type.substr(0, 8) == "smallint"
		  || type.substr(0, 7) == "tinyint")
    return "int";

  if (type.substr(0, 17) == "character varying") {
    t = "varchar";
    t.append(type.substr(17));
    return t;
  }
  
  if (type == "datetime")
   return "timestamp";

  // text is the same

  if(type.substr(0, 7) == "decimal") {
    t = "numeric";
    t.append(type.substr(7));
    return t;
  }

  // By default, everything is a text
  if (type == "UNKNOWN")
    return "text";

  return type;
}


uint64_t MySQLHelper::sequenceNextValue(DatabaseConnection *dbConn,
					const std::string &sequence) 
  throw (SQLException) 
{ 
  std::string query = "INSERT INTO " + sequence + " (\"dummy\") VALUES ('a');";
  DMESG("SQL: >" << query << "<" << std::endl);
  Statement statement(*dbConn);
  if(statement.exec(query)){
    throw SQLException(statement);
  }
  
  query = "DELETE FROM " + sequence + ";";
  DMESG("SQL: >" << query << "<" << std::endl);
  if(statement.exec(query)){
    throw SQLException(statement);    
  }
  
  query = "SELECT LAST_INSERT_ID();";
  DMESG("SQL: >" << query << "<" << std::endl);  
  uint64_t v;
  statement.bind(1, &v);
  if(statement.exec(query)){
    throw SQLException(statement);    
  }  
  if(statement.fetch()){
    throw SQLException(statement);    
  }
  return v;
}


uint64_t MySQLHelper::sequenceCurrentValue(DatabaseConnection *dbConn,
					   const std::string &sequence) 
  throw (SQLException) 
{
  Statement statement(*dbConn);
  std::string query = "SELECT LAST_INSERT_ID();";
  DMESG("SQL: >" << query << "<" << std::endl);  
  uint64_t v;
  statement.bind(1, &v);
  if(statement.exec(query)){
    throw SQLException(statement);    
  }  
  if(statement.fetch()){
    throw SQLException(statement);    
  }
  return v;
}

std::string MySQLHelper::readGUID(const std::string &guid) const 
{
  return "hex(\"" + guid + "\")";
}

std::string MySQLHelper::writeGUID(const std::string &guid) const
{
  return "unhex('" + guid + "')";
}
