/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "OracleHelper.h"
#include <cstring>
#include <stdlib.h>

std::string OracleHelper::mdToDBType(const std::string &type) const
{
  std::string t;

  if(type == "int")
    return "integer";

  if(type == "float")
    return "double precision";

  if(type.substr(0, 7) == "varchar") {
    t = "character varying";
    t.append(type.substr(7));
    return t;
  }

  if(type == "text")
    return "clob";

  return type;
}
 

std::string OracleHelper::dbToMDType(const std::string &type) const
{
  std::string t;

  if (type == "double precision")
    return "float";

  if (type == "integer")
    return "int";

  if (type.substr(0, 17) == "character varying") {
    t = "varchar";
    t.append(type.substr(17));
    return t;
  }

  if (type == "timestamp(6)")
    return "timestamp";

  if(type == "long")
    return "text";

  if(type == "clob")
    return "text";

  if (type.substr(0, 6) == "number") {
    t = "numeric";
    t.append(type.substr(6));
    return t;
  }
  
  return type;
}

//! Translates the results for the data description in Oracle into ints
//! Returns -1 if not a floating point number in buff
int OracleHelper::toInt(const char *buff)
{
  if(!strlen(buff))
     return -1;
  char *ptr;
  double d=strtod(buff, &ptr);
  if(ptr==buff)
    return -1;

  return (int)d;
}

std::string OracleHelper::readGUID(const std::string &guid) const 
{
  //  return "RAWTOHEX(\"" + guid + "\")";
  return "\"" + guid + "\"";
}

std::string OracleHelper::writeGUID(const std::string &guid) const
{
  return "HEXTORAW('" + guid + "')";
}
