/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef SITEMANAGER_H_
#define SITEMANAGER_H_

#include <string>
#include <vector>

#include "util/DBUtil.h"
#include "server/Constants.h"
#include "util/Debuggable.h"
#include "replicationdaemon/Exceptions.h"

class DatabaseConnection;
class MDStandalone;

struct Site {
    // Must use char[] so it can be sent over message queues, which
    // require struct with no heap allocated fields
    int id;
    char name[SITE_NAME_MAX_SIZE];
    char hostname[USERNAME_MAX_SIZE];
    int port;
    
    char login[USERNAME_MAX_SIZE];
    char password[PASSWORD_MAX_SIZE];
    bool useSSL;
    bool authenticateWithCertificate;
    char certFile[DIRNAME_MAX_SIZE];
    char keyFile[DIRNAME_MAX_SIZE];
    bool useGridProxy;
    bool verifyServerCert;
    char trustedCertDir[DIRNAME_MAX_SIZE];
    bool requireDataEncryption;
    char certPassword[PASSWORD_MAX_SIZE];
    
    friend std::ostream& operator<<(std::ostream& out, const Site& settings);
};
std::ostream& operator<<(std::ostream& out, const Site& settings);


class SiteManager : public Debuggable
{
public:
  SiteManager(DatabaseConnection &dbConn, bool debug = false);
	virtual ~SiteManager();

  void add(const std::string &site, const std::string &host, int port, 
    MDStandalone &server)
    throw (SQLException);
    
  void remove(const std::string &site)
    throw (SQLException, ReplicationException);

  void getAll(std::vector<Site> &sites)
    throw (SQLException);

  void getSite(const std::string &siteName, Site &site)
    throw (SQLException);
  void getSites(const std::vector<std::string> &siteNames, std::vector<Site> &sites)
    throw (SQLException);
    
  bool isSiteKnown(const std::string &siteName) 
    throw (SQLException);
    
  void setProperties(const std::string &site, 
        const std::vector<std::string> &properties, 
        const std::vector<std::string> &values)
    throw (SQLException);
    
  void getProperties(const std::string &site, 
        const std::vector<std::string> &properties, std::vector<std::string> &values)
    throw (SQLException);

private:
  void checkPropertyExists(const std::string &property)
    throw (SQLException);
    
  void readSites(const std::string &whereClause, std::vector<Site> &sites) 
    throw (SQLException);
    
  DatabaseConnection &dbConn;
};
#endif /*SITEMANAGER_H_*/
