/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: VOMSAttrUserManager.h,v 1.9 2010/04/14 07:15:30 asunil Exp $
//
/** VOMSUserManager is a class to which connects to 
    which maps certificate subjects to a user name.
    This user information is read-only.
**/

#ifndef VOMS_ATTR_USER_MANAGER_H
#define VOMS_ATTR_USER_MANAGER_H

#include "MDUserManager.h"

#include <map>

#define TUSER 1
#define TGROUP 2

class DatabaseConnection;

class VOMSAttrUserManager: public MDUserManager {
 public:
  VOMSAttrUserManager();
  VOMSAttrUserManager(DatabaseConnection *dbConn);
  virtual ~VOMSAttrUserManager();

  void setConnection(DatabaseConnection *dbConn);

  bool checkSubject(const std::string &user,
                     const std::string &subject, int type) const;

  bool checkVomsGroup(const std::string &user,
                     const std::string &subject) const;

  int allowVO(const std::string &vo);
  int disallowVO(const std::string &vo);

  int mapGroup(const std::string &groupMapping);
  int mapUser(const std::string &usrMapping);

  virtual bool checkCert(const std::string &user, X509 *cert,
			 std::string &vo, std::list< std::string > &groups,
			 std::string &eUser) const;

  std::string defaultUser(const std::string &vo) const;

  std::string getMappedGroup(const std::string voGroup) const;

  std::string getMappedUser(const std::string voRole) const;

  virtual int listUsers(std::list < std::string> &users);

  virtual int listUserCred(const std::string &user, UserCred &cred);

  int addUserVoms(const std::string &user,
                  const std::string &subject, int type);
  int removeUserVoms(const std::string &subject, int type);
  int listUserVoms(const std::string &result, int type);


 private:
  int parseMapping(const std::string mapping,
		   std::string &external,
		   std::string &internal) const;
  
  std::list < std::string > allowedVOs;

  // Maps a default user to a VO
  std::map < std::string, std::string > defaultUserMap;

  // Maps groups to a VO group
  std::map < std::string, std::string > groupMap;

  // Maps users to a VO role
  std::map < std::string, std::string > userMap;

  DatabaseConnection *dbConn;

};

#endif //VOMS_USER_MANAGER_H
