/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: VOMSUserManager.h,v 1.4 2010/04/14 07:15:30 asunil Exp $
//
/** VOMSUserManager is a class to which connects to 
    which maps certificate subjects to a user name.
    This user information is read-only.
**/

#ifndef VOMS_USER_MANAGER_H
#define VOMS_USER_MANAGER_H

#include "MDUserManager.h"
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <map>

class VOMSUserManager: public MDUserManager {
 public:
  VOMSUserManager();
  virtual ~VOMSUserManager();

  int init(const std::string &cert,
	   const std::string &key);

  int addVOMS(const std::string &URL,
	      const std::string &name);

  virtual bool checkSubject(const std::string &user,
			    const std::string &subject) const;
  
  virtual bool isKnownSubject(const std::string &subject) const;

  virtual int listUsers(std::list < std::string > &users);

  virtual int listUserCred(const std::string &user, UserCred &cred);

 private:
  std::map< std::string, std::string > certificates;

  int connectToServer(const std::string &hostname,
		      int port);
  int disconnect();

  int initSSL(const std::string &cert, const std::string &key);

  int sendRequest(const std::string &m);

  int readResponse(std::string &m);

  int parseURL(const std::string &URL, std::string &hostname,
	       std::string &path, int &port);

  int doGetRequest(const std::string &request,
		   std::string &result);

  int parseXML(const std::string &xmlList, const std::string &group);

  std::string hostcert;
  std::string hostkey;
  int sock;   // Client socket
  SSL_CTX *ctx;  // SSL server context
  SSL *ssl;
};

#endif //VOMS_USER_MANAGER_H
