/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
// $Id: protocolHandling.h,v 1.9 2010/11/08 07:40:39 asunil Exp $

#ifndef PROTOCOL_HANDLING_H
#define PROTOCOL_HANDLING_H

#include "server/UserContext.h"
#include "util/Socket.h"

#include <list>
#include <string>
#include <stdint.h>

#include <openssl/ssl.h>

class AuthContext;
class MDServer;
class MDUserManager;
class ConfigParser;

//int readLine(int ipConnection, SSL *ssl,
//	     std::string &line, std::string &buffer);
int initConnection(TCPSocket &socket, bool advertiseSSL, const std::string greeting,
            bool &useSSL, uint64_t &sessionID) throw (SocketException);
void establishUserContext(TCPSocket &socket, struct UserContext &context) 
            throw (SocketException);

bool authenticateUser( SSL *ssl, MDServer *connection, 
		       const AuthContext &authCtx, UserContext &context);

void createUserManagers(
    const ConfigParser &configParser, 
    std::list<MDUserManager *> &userManagers);

void removeUserManagers( std::list<MDUserManager *> &userManagers ) ;

     
/**
 * Escapes a string so that it can be transmitted between
 * the client and the server. It does the following replacements:
 * 
 * ' -> \' 
 * \ -> \\
 * 
 * This is called by the dump command (which needs to generate
 * commands that can be sent directly to the mdclient).
 */
void escapeString(std::string &str);

std::string escapeString(char *str);

/**
 * Removes escape characters from the string. This is the reverse 
 * of addEscapeCoding() and should be called by the mdinterpreter.
 * 
 * \' -> '
 * \\ -> \
 */
void unescapeString(std::string &str);
void unescapePreparedString(std::string &str);

#endif // PROTOCOL_HANDLING_H
