/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "CoreDataAccess.h"
#include "debug.h"

CoreDataAccess::CoreDataAccess(const std::string &host , int port, 
            const std::string &path,
            const std::string &configFile, bool keep_alive, bool dbg)
         : MDSoapClient(host, port, path, configFile, keep_alive, dbg)
{
}

int CoreDataAccess::MakeDestroyDataResourceRequest(std::string &dran
                     , DestroyDataResourceRequest **req) 
{

  DestroyDataResourceRequest* seReq 
      = soap_new__wsdai__DestroyDataResourceRequest(&soap,-1);
  *req = seReq;

  seReq->wsdai__DataResourceAbstractName = dran;

  DMESG("DRAN : " << seReq->wsdai__DataResourceAbstractName << endl);

  return SOAP_OK;
}

int CoreDataAccess::DestroyDataResource(DestroyDataResourceRequest *req)
{
  setUser();

  DestroyDataResourceResponse* res 
    = soap_new__wsdai__DestroyDataResourceResponse(&soap,-1);

  int ret = soap_call___ns6__DestroyDataResource(&soap
            , access_end_point.c_str(), NULL, req, res);

  if (ret != SOAP_OK ) {
    errorMsg = soap.fault->faultstring;
    return ret;
  }
  return SOAP_OK;
}

int CoreDataAccess::SimpleExecuteDestroy(std::string &dran)
{
  DestroyDataResourceRequest*  seReq ;

  int error = 0 ;
  
  do {
    int ret = MakeDestroyDataResourceRequest(dran, & seReq);

    ret = DestroyDataResource(seReq);
    if( ret != SOAP_OK) { error = 1; break; }

  } while (0) ;

  if ( error == 1 ) {
    cout << "Error : " << errorMsg << endl ;
    return SOAP_FAULT;
  } else {
    cout << "Success !!" << endl ;
  }
  return SOAP_OK;
}


