/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "CoreResourceList.h"
#include "debug.h"

CoreResourceList::CoreResourceList(const std::string &host , int port, 
            const std::string &path,
            const std::string &configFile, bool keep_alive, bool dbg)
         : MDSoapClient(host, port, path, configFile, keep_alive, dbg)
{
}

int CoreResourceList::GetDataResourceList(GetDataResourceListRequest *req,
                          GetDataResourceListResponse **res)
{
  setUser();

  *res = soap_new__wsdai__GetResourceListResponse(&soap, -1);

  int ret =  soap_call___ns7__GetResourceList(&soap
            , access_end_point.c_str(), NULL, req, *res);

  if (ret != SOAP_OK ) {
    errorMsg = soap.fault->faultstring;
    return ret;
  }
  return SOAP_OK;
}

int CoreResourceList::MakeDataResourceListRequest(GetDataResourceListRequest **req) 
{
  GetDataResourceListRequest*  seReq 
     = soap_new__wsdai__GetResourceListRequest(&soap,-1);
  *req = seReq;

  return SOAP_OK;
}

int CoreResourceList::SimpleExecuteList()
{
  GetDataResourceListRequest*  seReq ;
  GetDataResourceListResponse* seRes ;

  int error = 0 ;
  
  do {
    int ret = MakeDataResourceListRequest(& seReq);
    if (ret != SOAP_OK) { error = 1; break; }

    ret = GetDataResourceList(seReq, &seRes);
    if( ret != SOAP_OK) { error = 1; break; }


    int size = seRes->wsdai__DataResourceAddress.size();
   
    EndpointReference* er;
    for(int i=0; i<size;i++){
      er = seRes->wsdai__DataResourceAddress[i];
      PrintEndpointReference( er );
    }

  } while (0) ;

  if ( error == 1 ) {
    cout << "Error : " << errorMsg << endl ;
    return SOAP_FAULT;
  } 

  return SOAP_OK;
}

void dran_deserialize( std::string & dran, std::string & rp_dran)
{
  struct soap *ctx = soap_new1(SOAP_XML_STRICT);  ;
  std::string str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  str.append(dran);
  istringstream *in = new istringstream(str, istringstream::in);


  ctx->is = in;

  wsdai__DataResourceAbstractNameType__ wsdai_dran;
  soap_begin_recv(ctx);
  wsdai_dran.soap_get(ctx, "DataResourceAbstractName", NULL); 
  soap_end_recv(ctx);
  soap_destroy(ctx);
  soap_end(ctx);
  soap_free(ctx);
  rp_dran = wsdai_dran.__item ;
}


void CoreResourceList::PrintEndpointReference(EndpointReference* er)
{
  if ( er == NULL ) return ;
  if (er->Address != NULL )  {
    DMESG("EndPoint : " << er->Address->__item << std::endl);
    std::cout << "Address : " << er->Address->__item << std::endl;
  }
  if (er->wsa__ReferenceParameters != NULL )  {
    // DMESG("DRAN : " << er->wsa__ReferenceParameters->__item << std::endl);
    std::string rp_dran;
    std::string str = er->wsa__ReferenceParameters->__item; 
    dran_deserialize(str, rp_dran);
    std::cout << "DRAN : " << rp_dran << std::endl;
  }
}


