/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef DBCONNECTIONPOOL_H
#define DBCONNECTIONPOOL_H
#include "DBConnection.h"

#include <vector>
#include <boost/thread/condition.hpp>

class ConfigParser;

#define DEFAULT_N_CONNECTIONS 10

class DBConnectionPool{
public:
  /** This object is the owner of config, it will delete it in the destructor.
   */
  DBConnectionPool(ConfigParser *config, size_t nConnections = DEFAULT_N_CONNECTIONS);
  virtual ~DBConnectionPool();

  /**
   * Returns null if no connection is available.
   */
  DBConnection* checkoutConnection();

  /**
   */
  void checkinConnection(DBConnection *dbConn);

  /** Creates a new connection to replace one
	* that may have been cancelled or failed due to some other reason
	*/
  void createNewConnection();

  /**
   * Returns the number of available connections on the
   * connection pool
   */
  size_t availableConnections();

private:
	size_t _poolSize;
	std::vector<DBConnection *> _pool;

	boost::mutex monitor;	  
	ConfigParser *_config;
};

#endif // DBCONNECTIONPOOL_H
