/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef MDSOAPCLIENT_H 
#define MDSOAPCLIENT_H 

#include <openssl/ssl.h>
#include <openssl/err.h> 
                           
#include <string>
#include <list>
#include <stdint.h>

#include "stdsoap2.h"

#define MDSOAP_DEFAULT_PORT 8833
#define SOAP_DATA_FAULT -999

#include "WSDairH.h"

typedef _wrs__webRowSet WebRowSet;

using namespace std;

class ConfigParser;

class MDSoapClient; 
void int_sig_handler(int sig); 
MDSoapClient *registerClient(MDSoapClient* client); 

class MDSoapClient { 
  friend void int_sig_handler(int); 
  friend MDSoapClient *registerClient(MDSoapClient *); 

public:
  MDSoapClient(const std::string &host, int port, const std::string &path,
               const std::string &configFile, bool keep_alive, bool dbg);
  ~MDSoapClient();
  int execute(std::string, std::string, WebRowSet**, int&);
  std::string getError();
  void freeSoap();
  void print( _wrs__webRowSet*);
  int initsoapSSL();
  int loadCertificate();
  int establishContext();
  void setUser();

  void setHost(std::string & host) { _host = host; }
  void setPort(int port) { _port = port; }
  void setEndPoint(std::string & endPoint) { access_end_point = endPoint; }

protected:
  bool debug;
  struct soap soap;

  ConfigParser *config;
  bool useSSL;
  bool requireSSL;
  bool verifyServerCert;
  static char certKey[256];
  int authWithCert;

  std::string certFile;
  std::string keyFile;

  volatile bool cancellationRequested;
  static MDSoapClient *clientRegisteredForInt;

  std::string _host ;
  int _port;
  std::string _path ;
  std::string access_end_point;
  std::string _userid;
  std::string _passwd;

  std::string errorMsg;

};




#endif /*MDSOAPCLIENT_H_*/



