/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef MDSOAPSERVER_H
#define MDSOAPSERVER_H

#include <pthread.h>
#include <vector>

#include "configparser/ConfigParser.h"
#include "DBConnection.h"
#include "DBConnectionPool.h"
#include "SessionManager.h"
 
#include "util/BoundedQueue.h"
#include "server/MDInterpreter.h"

#include "WSDairH.h"

#include "util/SecurityUtils.h"
#include "server/protocolHandling.h"

#include "replicationdaemon/ReplicationManager.h"



class MDSoapServer{
public:
	MDSoapServer(int port, ConfigParser &configParser);
	~MDSoapServer();
	void startServing();
	void stopServing();

	static void setDebug(bool _debug);
	static bool getDebug();

private:
	BoundedQueue<struct soap*> _taskQueue;
	int _port;

	pthread_t _master;
	std::vector<pthread_t> _workers;

	DBConnectionPool *_dbConnPool;
	// Holds the sessions for streaming operations.
	SessionManager *_sessionManager;

//	MetadataServiceImpl _soap;
	struct soap _soap;
	//   struct ThreadContext ctx;
	bool _soapKeepAlive;
  
	int _useSSL;
        int _requireAuthentication ;
        int _allowCertAuth ;
        int _allowGridProxyLogin ;
        int _allowPasswordAuthentication ;
        int _allowMyProxyHack ;
        std::string _serverCertKeyFile ;
        std::string _trustedCertDir ;

	void initSSL();
};


#endif // MDSOAPSERVER_H
