/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef SQLACCESS_H
#define SQLACCESS_H

#include "MDSoapClient.h"

#define _NO_OPS 0
#define _SELECT 1
#define _INSERT 2
#define _DELETE 3
#define _UPDATE 4

#define WRS_FORMAT_URI "http://java.sun.com/xml/ns/jdbc"

typedef _wsdair__SQLExecuteRequest  SQLExecuteRequest;
typedef _wsdair__SQLExecuteResponse SQLExecuteResponse;
typedef _wsdai__GetDataResourcePropertyDocumentRequest GetSQLPropertyDocumentRequest ;
typedef wsdair__SQLPropertyDocumentType  GetSQLPropertyDocumentResponse;
typedef wsdai__PropertyDocumentType PropertyDocument;
typedef _wrs__webRowSet WebRowSet;

class SQLAccess: public MDSoapClient {
public:
  SQLAccess(const std::string &host="" , int port=8833, const std::string &path = "",
            const std::string &configFile = "", bool keep_alive=false, bool dbg=false);

  ~SQLAccess();

  int GetSQLPropertyDocument(GetSQLPropertyDocumentRequest *req
                           , GetSQLPropertyDocumentResponse **res);

  int MakeGetSQLPropertyDocumentRequest(std::string &dran
                     , GetSQLPropertyDocumentRequest **req);

  int SimplePropertyDocument(std::string &dran);
 
  void PrintDatasetMap(PropertyDocument* res);
  void PrintDefaultConfiguration(wsdai__ConfigurationDocumentType *cdt);
  void PrintConfigurationMap(PropertyDocument* res);
  void PrintLanguageMap(PropertyDocument* res);
  void PrintGeneralInfo(PropertyDocument* res);
  void PrintPropertyDocument(PropertyDocument* res);


  int SQLExecute(SQLExecuteRequest *req,
                 SQLExecuteResponse **res);


  int FetchWRS(SQLExecuteResponse *seRes
                      , WebRowSet** wrs) ;

  int GetUpdateCount( SQLExecuteResponse *seRes
                             , int& updateCount);

  int MakeSQLExecuteRequest(std::string &dran
                     , std::string &uri
                     , std::string &command
                     , SQLExecuteRequest **req) ;


  int SimpleExecute(std::string &dran
                  , std::string &ommand
                  , const std::string uri = "");


  void PrintWRS(WebRowSet* wrs, int type);

protected:

private:
  int parseType(std::string &command);
};

#endif // SQLACCESS_H
