/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "SQLAccessFactory.h"
#include "debug.h"

SQLAccessFactory::SQLAccessFactory(const std::string &host , int port, 
         const std::string &path,
         const std::string &configFile, bool keep_alive, bool dbg)
         : MDSoapClient(host, port, path, configFile, keep_alive, dbg)
{
}

int SQLAccessFactory::SQLExecuteFactory(SQLExecuteFactoryRequest *req,
                          SQLExecuteFactoryResponse **res)
{
  setUser();

  *res = soap_new_wsdai__DataResourceAddressListType(&soap, -1);

  int ret =  soap_call___ns2__SQLExecuteFactory(&soap
            , access_end_point.c_str(), NULL, req, *res);

  if (ret != SOAP_OK ) {
    errorMsg = soap.fault->faultstring;
    return ret;
  }
  return SOAP_OK;
}

int SQLAccessFactory::MakeSQLExecuteFactoryRequest(std::string &dran
                     , std::string &command
                     , SQLExecuteFactoryRequest **req)
{
  SQLExecuteFactoryRequest*  seReq 
     = soap_new__wsdair__SQLExecuteFactoryRequest(&soap,-1);
  *req = seReq;

  seReq->wsdair__SQLExpression = soap_new_wsdair__SQLExpressionType(&soap, -1) ;
  seReq->wsdair__SQLExpression->Expression = command ;
  seReq->wsdai__DataResourceAbstractName = dran;

  DMESG("DRAN : " << seReq->wsdai__DataResourceAbstractName << endl);
  DMESG("EXPR : " << seReq->wsdair__SQLExpression->Expression << endl);

  return SOAP_OK;
}

int SQLAccessFactory::FetchEndpointReference(SQLExecuteFactoryResponse *seRes
                      , EndpointReference** er)
{
  if ( (seRes->wsdai__DataResourceAddress.size() == 0 ) ||
       (seRes->wsdai__DataResourceAddress[0] == NULL ))  {
    errorMsg = "No Endpoint Reference";
    return SOAP_DATA_FAULT;
  }

  *er = seRes->wsdai__DataResourceAddress[0];

  return SOAP_OK;
}

extern int dran_deserialize( std::string & dran, std::string & rp_dran);
/*
int dran_deserialize( std::string & dran, std::string & rp_dran)
{
  struct soap *ctx = soap_new1(SOAP_XML_STRICT);  ;
  std::string str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  str.append(dran);
  istringstream *in = new istringstream(str, istringstream::in);


  ctx->is = in;

  wsdai__DataResourceAbstractNameType__ wsdai_dran;
  soap_begin_recv(ctx);
  wsdai_dran.soap_get(ctx, "DataResourceAbstractName", NULL);
  soap_end_recv(ctx);
  soap_destroy(ctx);
  soap_end(ctx);
  soap_free(ctx);
  rp_dran = wsdai_dran.__item ;
}
*/


void SQLAccessFactory::PrintEndpointReference(EndpointReference* er,
    std::string &result)
{
  if ( er == NULL ) return ;
  if (er->Address != NULL )  {
    DMESG("EndPoint : " << er->Address->__item << std::endl);
    std::cout << er->Address->__item << std::endl;
  }

  if (er->wsa__ReferenceParameters != NULL )  {
    std::string rp_dran;
    std::string str = er->wsa__ReferenceParameters->__item;
    dran_deserialize(str, rp_dran);
    std::cout << "DRAN : " << rp_dran << std::endl;
    result = rp_dran;
  }

}

int SQLAccessFactory::SimpleExecute(std::string dran
                     , std::string command, std::string& result)
{                    
  SQLExecuteFactoryRequest*  seReq ;
  SQLExecuteFactoryResponse* seRes ;
  
  int error = 0 ;
  
  do {
    int ret = MakeSQLExecuteFactoryRequest(dran, command, & seReq);
    if (ret != SOAP_OK) { error = 1; break; }
    
    ret = SQLExecuteFactory(seReq, &seRes);
    if( ret != SOAP_OK) { error = 1; break; }
    
    EndpointReference* er;
    ret =  FetchEndpointReference(seRes, & er) ;
    if (ret != SOAP_OK) { error = 1 ; break; }

    PrintEndpointReference( er, result );
    break;
  } while (0) ;

  if ( error == 1 ) {
    cout << "Error : " << errorMsg << endl ;
    return SOAP_FAULT;
  }

  return SOAP_OK;
}


