/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "SQLResponseFactory.h"
#include "debug.h"

SQLResponseFactory::SQLResponseFactory(const std::string &host , int port,
         const  std::string &path,
         const std::string &configFile, bool keep_alive, bool dbg)
         : MDSoapClient(host, port, path, configFile, keep_alive, dbg)
{
}

int SQLResponseFactory::SQLRowsetFactory(SQLRowsetFactoryRequest *req,
                          SQLRowsetFactoryResponse **res)
{
  setUser();

  *res = soap_new_wsdai__DataResourceAddressListType(&soap, -1);

  int ret =  soap_call___ns4__GetSQLRowsetFactory(&soap
            , access_end_point.c_str(), NULL, req, *res);

  if (ret != SOAP_OK ) {
    errorMsg = soap.fault->faultstring;
    return ret;
  }
  return SOAP_OK;
}

int SQLResponseFactory::MakeSQLRowsetFactoryRequest(std::string &dran
                     , int position, int count
                     , SQLRowsetFactoryRequest **req)
{
  SQLRowsetFactoryRequest*  seReq 
     = soap_new__wsdair__GetSQLRowsetFactoryRequest(&soap,-1);
  *req = seReq;

  seReq->wsdai__DataResourceAbstractName = dran;
  seReq->Position = position;
  seReq->Count = (unsigned int*) soap_new_xsd__unsignedInt(&soap,-1);
  *(seReq->Count) = count;

  return SOAP_OK;
}

int SQLResponseFactory::FetchEndpointReference(SQLRowsetFactoryResponse *seRes
                      , EndpointReference** er)
{
  if ( (seRes->wsdai__DataResourceAddress.size() == 0 ) ||
       (seRes->wsdai__DataResourceAddress[0] == NULL ))  {
    errorMsg = "No Endpoint Reference";
    return SOAP_DATA_FAULT;
  }

  *er = seRes->wsdai__DataResourceAddress[0];

  return SOAP_OK;
}

extern int dran_deserialize( std::string & dran, std::string & rp_dran);


void SQLResponseFactory::PrintEndpointReference(EndpointReference* er,
        std::string & result)
{
  if ( er == NULL ) return ;
  if (er->Address != NULL )  {
    DMESG("EndPoint : " << er->Address->__item << std::endl);
    std::cout << er->Address->__item << std::endl;
  }

  if (er->wsa__ReferenceParameters != NULL )  {
    std::string rp_dran;
    std::string str = er->wsa__ReferenceParameters->__item;
    dran_deserialize(str, rp_dran);
    std::cout << "DRAN : " << rp_dran << std::endl;
    result = rp_dran;
  }

}

int SQLResponseFactory::SimpleExecute(std::string dran
                     , int position, int count, std::string &result)
{                    
  SQLRowsetFactoryRequest*  seReq ;
  SQLRowsetFactoryResponse* seRes ;
  
  int error = 0 ;
  
  do {
    int ret = MakeSQLRowsetFactoryRequest(dran, position, count, & seReq);
    if (ret != SOAP_OK) { error = 1; break; }
    
    ret = SQLRowsetFactory(seReq, &seRes);
    if( ret != SOAP_OK) { error = 1; break; }
    
    EndpointReference* er;
    ret =  FetchEndpointReference(seRes, & er) ;
    if (ret != SOAP_OK) { error = 1 ; break; }

    PrintEndpointReference( er, result );
    break;
  } while (0) ;

  if ( error == 1 ) {
    cout << "Error : " << errorMsg << endl ;
    return SOAP_FAULT;
  }

  return SOAP_OK;
}


