/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "S_SQLAccessFactory.h"
#include "util/Statement.h"
#include "DBResult.h"
#include "util/SecurityUtils.h"

static bool debug = false;
#include "debug.h"

void S_SQLAccessFactory::init(struct soap* soap
                            , AutoDBConnection *helper
                            , std::string &dran
                            , std::string &query
                            , bool _debug) 
{
    this->soap = soap;
    this->dran = dran;
    this->query = query;
    this->helper = helper;
    debug = _debug;
}


int S_SQLAccessFactory::commitDirectoryToIndex2(EntryProps &prop
                           , const std::string &indexTable
                           , std::string &table
                           , Statement &statement)
{ 
  std::ostringstream query;
  
  query << "UPDATE " << indexTable 
        << " SET \"table_name\" = '" << table  << "', "
        << " \"directory\" = '" << dran << "',"
        << " \"flags\" = '" << prop.flags << "'"
        << " WHERE \"id\"=" << prop.id
        << ";";

  DMESG("SQL: >" << query.str() << "<" << std::endl);
  
  if(statement.exec(query.str())){
    DMESG("16 Directory exists\n");
    return -1;
  } 

  return prop.id;
}

class AuthContext;
extern AuthContext _authCtx;

int S_SQLAccessFactory::checkTooManyIndirectServiceForUser()
{

  std::ostringstream query;

  std::string login;	/* user name */
  int count; 		/* number of indirect services created by the user */

  Statement statement(*(helper->dbConn->dbServer->dbConn), debug);

  if ( soap->userid != NULL ) {
    login = soap->userid;
  } else {
    login = "";
  }

  query << "select count(*) from masterindex "
        << "where \"directory\" like '/SQLResponse/response%' "
        << "and \"flags\" != '0' and \"owner_name\" = '"
        << login
        << "' ;";

  // DMESG("SQL CHeck: >" << query.str() << "<" << std::endl);

  if ( statement.exec(query.str()) )
    return servBusy(soap, "DB Error");

  statement.bind(1, &count);
  if(statement.fetch()){
    return servBusy(soap, "DB Error");
  }
  statement.unBind();

  // DMESG("Count : >" << count  << "<" << std::endl);

  if ( (_authCtx.indirectServiceLimit > 0)  
      &&  ( count > _authCtx.indirectServiceLimit)  ) {
    return servBusy(soap, "Too Many Indirect Services created !!");
  }
  return SOAP_OK;
}

int S_SQLAccessFactory::handleQuery()
{
  // get what command is requested
  MDInterpreter interpreter;
  MDInterpreter::CommandToken cmdToken = interpreter.parseCommand (query);

  // check whether the command is supported
  int ret = checkInvalidCommand(soap ,cmdToken, _ACCESS_FACTORY);
  if ( ret != SOAP_OK ) return ret;

  ret = checkDBConnection(soap, helper);
  if (ret != SOAP_OK ) return ret;

  ret = checkTooManyIndirectServiceForUser();
  if ( ret != SOAP_OK ) return ret;
  
  Statement statement(*(helper->dbConn->dbServer->dbConn), debug);

  // initialze prop
  EntryProps prop;
  prop.directoryTable = "SQLRespone";
  prop.indexTable = "masterindex";
  prop.flags = 0;

  std::string table;

  int error = 0 ;
  do {
    if( statement.beginTransaction()) { error = 1; break; }

    // parse requested query
    std::string translatedQuery;
    helper->dbConn->dbServer->sqlParse(query, cmdToken, translatedQuery);

    int isPlain = helper->dbConn->dbServer->asp->isPlain ;

    DBResult dbResult;
    std::vector<std::string> results;
    helper->dbConn->getResult(dbResult,results);
    if (dbResult.errorCode != 0) {   // error
      return returnErrorDBResult(soap, dbResult);
    }

    std::stringstream query_s ;

    // add an directory entry into masterindex
    prop.id = helper->dbConn->dbServer->addDirectoryToIndex("SQLResponse"
                , prop, statement);
    if ( prop.id < 0 ) { error = 1; break; }

    switch(cmdToken) {
      case MDInterpreter::cSELECT:
      {
        table = "responses" + StringOf(prop.id) + "_"
                 + StringOf(helper->dbConn->dbServer->asp->numColumn) ;
        query_s << "CREATE VIEW " << table << " AS " << translatedQuery << ";";
        break;
      }
      case MDInterpreter::cUPDATE:
      case MDInterpreter::cDELETE:
      case MDInterpreter::cINSERT:
      {
        query_s <<  translatedQuery << ";";
        table = "responseu" + StringOf(prop.id) + "_"  ;
        break;
      }
      default:
        ERRLOG("Unsupported command token: " << cmdToken);  
    }
    DMESG("Translated query : " <<  query_s.str() << std::endl);

    if( statement.exec(query_s.str())) { error = 1; break; }

    switch(cmdToken) {
      case MDInterpreter::cUPDATE:
      case MDInterpreter::cDELETE:
      case MDInterpreter::cINSERT:
      {
        table += StringOf(statement.numRows() ) ;
        break;
      }
      default:
        ERRLOG("Unsupported command token: " << cmdToken);      
    }

    dran = "/SQLResponse/" + table ;
    prop.flags = EPT_DIR ;
    if( isPlain ) prop.flags += EP_PLAIN;
    prop.id = commitDirectoryToIndex2(prop, "masterindex", table, statement);
  } while ( false ) ;


  if (error)  {
    return generateSoapError(soap, "Invalid Expression Fault" , NULL
            , SOAP_TYPE_wsdai__InvalidExpressionFaultType );
  }

  if ( helper->dbConn->dbServer->finalizeDirectoryToIndex(prop.id, statement) < 0) {
    return generateSoapError(soap, "Invalid Expression Fault" , NULL
                , SOAP_TYPE_wsdai__InvalidExpressionFaultType );
  }

  statement.commitTransaction();

  return SOAP_OK;
}

void S_SQLAccessFactory::makeResponse(wsdai__DataResourceAddressListType* res)
{
  wsdai__DataResourceAddressType *er = makeEndPoint(soap, dran, "SQLResponse");
  res->wsdai__DataResourceAddress.push_back(( wsdai__DataResourceAddressType*) er);
}


