/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "S_SQLResponse.h"
#include "util/Statement.h"
#include "server/DBHelper.h"

using namespace std;

static bool debug = false;
#include "debug.h"

typedef std::list<EntryProps> EntryPropList;


int S_SQLResponse::makePropertyDocument( bool update
        , wsdair__SQLResponsePropertyDocumentType* doc)
{ 
  std::string endpoint ;
  std::string address ;
  transEndPoint(soap, address, endpoint, "SQLResponse", dran);
  doc->wsdai__DataResourceAbstractName = endpoint;
  doc->wsdai__DataResourceManagement = _wsdai__DataResourceManagement__ExternallyManaged;

  /* Parent Data Resource */

  // doc->wsdai__ParentDataResource  
  //    = (wsdai__DataResourceAddressType*) makeEndPoint(soap, DEFAULT_DRAN, "SQLAccess");

    
  /* Datamap */

  wsdai__DatasetMapType *dmt = soap_new_wsdai__DatasetMapType(soap,-1);
  dmt->wsdai__MessageQName = "wsdair:GetSQLRowset" ;
  dmt->wsdai__DatasetFormatURI = WRS_FORMAT_URI;
  doc->wsdai__DatasetMap.push_back(dmt);
  dmt = soap_new_wsdai__DatasetMapType(soap,-1);
  dmt->wsdai__MessageQName = "wsdair:GetSQLResponseItem" ;
  dmt->wsdai__DatasetFormatURI = WRS_FORMAT_URI;
  doc->wsdai__DatasetMap.push_back(dmt);
    
  dmt = soap_new_wsdai__DatasetMapType(soap,-1);
  dmt->wsdai__MessageQName = "wsdair:GetSQLResponseItem" ;
  dmt->wsdai__DatasetFormatURI = WRS_FORMAT_URI;
  doc->wsdai__DatasetMap.push_back(dmt);
    
  _wsdai__DataResourceDescription *drd = soap_new__wsdai__DataResourceDescription(soap,-1)
;     
  doc->wsdai__DataResourceDescription = drd;
  drd->__mixed = soap_strdup(soap, "AMGA");
                        
  doc->wsdai__Readable = true;
  doc->wsdai__Writeable = true;
  doc->wsdai__ConcurrentAccess = true;

  doc->wsdai__TransactionInitiation = _wsdai__TransactionInitiation__Automatic;
  doc->wsdai__TransactionIsolation = _wsdai__TransactionIsolation__ReadCommitted;
  doc->wsdai__ChildSensitiveToParent = _wsdai__ChildSensitiveToParent__Sensitive;
  doc->wsdai__ParentSensitiveToChild = _wsdai__ParentSensitiveToChild__Sensitive;

  doc->wsdair__NumberOfSQLRowsets = 0;
  doc->wsdair__NumberOfSQLUpdateCounts = 0;
  doc->wsdair__NumberOfSQLReturnValues = 0;
  doc->wsdair__NumberOfSQLOutputParameters = 0;
  doc->wsdair__NumberOfSQLCommunicationsAreas = 0;

  wsdair__SQLResponseItemType *item = soap_new_wsdair__SQLResponseItemType(soap,-1);
  item->SequenceNumber = 0 ;
  if ( update ) {
    item->Name = _wsdair__SQLResponseItemType_Name__UpdateCount;
    doc->wsdair__NumberOfSQLUpdateCounts = 1;
  } else {
    item->Name = _wsdair__SQLResponseItemType_Name__Rowset;
    doc->wsdair__NumberOfSQLRowsets = 1;
  }
  doc->wsdair__SQLResponseItem.push_back(item);


  return SOAP_OK;
}


bool S_SQLResponse::isUpdate()
{
  size_t found = dran.find("responseu");
  if( found == std::string::npos ) { return false; }
  else { return true; }
}


int S_SQLResponse::parseDran(std::string &table,
                          char &type,
                          int &num_column_or_row, int rowset_type)
{

  trim2(dran);
  DMESG("requested DRAN >>" << dran << "<<" << endl);

  int error = 0;

  do {

    std:string temp_str ;
    if(rowset_type == 0 ) {
      temp_str = DIR_PREFIX;
    } else {
      temp_str = DIR_PREFIX2;
    }

    int temp_cnt = temp_str.size();
   
    if( dran.empty() ) {
      error = 1; break;
    }

    size_t pos = dran.find(temp_str);
    DMESG("Dir Prefix >>" << temp_str << "<<" << endl);
    DMESG("pos >>" << pos << "<<" << endl);

    if ( pos == std::string::npos ) return wrongDran(soap, dran);

    table = dran.substr(pos + temp_cnt);

    dran.clear();
    dran.append(DIR_PREFIX);
    dran.append(table);

    DMESG("requested table >>" << table << "<<" << endl);
    DMESG("Type >>" << dran[temp_cnt + 8]  << "<<" << endl);

    switch ( dran[temp_cnt + 8] ) {
      case 's' : type = 'S';   break;
      case 'u' : type = 'U';   break;
      default : error = 1; break;
    }

    pos = dran.find('_');
    if( pos == string::npos ) { error = 1; break; }

    std::string num = dran.substr(pos+1);
    DMESG("requested column or update number >>" << num << "<<" << endl);

    num_column_or_row = 0 ;
    for(size_t i=0; i < num.size(); i++) {
      if( num[i] < '0' || num[i] > '9' )
        { error = 1; break; }
      num_column_or_row = num_column_or_row * 10 + num[i] - '0' ;
    }

  } while (0);

  if( error ) {
    DMESG("Format of DRAN is Wrong " << endl);
    return wrongDran(soap, dran);
  }

  int ret = ValidDran();
  if( ret != SOAP_OK ) {
    DMESG("DRAN does not exists " << endl);
    return ret;
  }

  return SOAP_OK;
}


int S_SQLResponse::handleRequest( std::string &table,
                             int count,
                             int &num_column_row,
                             DBResult &dbResult,
                             std::vector<std::string> &results)
{
  std::ostringstream os;

  int ret = checkDBConnection(soap, helper);
  if (ret != SOAP_OK ) return ret;

  // std::string backend = helper->dbConn->dbServer->dbHelper->backend() ;

  os << "SELECT * FROM /SQLResponse/" << table ;
  if ( count != 0 )  {
    os << " LIMIT " << count << " OFFSET " << position ;
  }
  os << ";";

  DMESG("LIMIT : " << count << " OFFSET " << position  << endl);

  std::string queryS(os.str());
//  bool  foundSomething = false;

  helper->dbConn->dbServer->sqlExec(queryS, MDInterpreter::cSELECT);
  DMESG("Input Query : " << queryS << endl);

  helper->dbConn->getResult(dbResult,results);
  if (dbResult.errorCode != 0) {   // error
    return returnErrorDBResult(soap, dbResult);
  }
  DMESG("End of handleRequest "  << endl);

  overFlow = helper->dbConn->isOverFlow();

  return SOAP_OK;
}

int S_SQLResponse::makeResponse(int colNum
                              , vector<string> &results)
{
  int rowsNo = results.size() / colNum ;
  std::string cmd = "";
  Dataset->wsdai__DatasetData->wrs__webRowSet
      = createWRS(soap, colNum, rowsNo, overFlow
                , dran, cmd, results, MDInterpreter::cSELECT);
  if ( typeid(*Dataset) == typeid(wsdair__SQLDatasetType) ) {
    wsdair__SQLDatasetType* SQLDataset = (wsdair__SQLDatasetType*) Dataset;
    SQLDataset->wsdair__SQLUpdateCount.push_back(0);
  }
  return SOAP_OK;
}

int S_SQLResponse::processITEM()
{ 
  std::string table ;
  char type ;
  int num_column_or_row = 0;
  
  int count ;
  if ( opType != OP_TUPLE) {
    count = 0 ;
  } else if ( reqCount == NULL ) {
    count = 1;
  } else {
    count = *(reqCount);
    if( count < 0 ) count = 0 ;
  }
  
  int ret = parseDran(table, type, num_column_or_row);
  if( ret != SOAP_OK ) return ret;

  Dataset->wsdai__DatasetFormatURI = WRS_FORMAT_URI ;
  Dataset->wsdai__DatasetData = soap_new_wsdai__DatasetDataType(soap, -1);
  
  if( type == 'U') {   // update or delete or insert response
    switch(opType) {
      case OP_ITEM:
      case OP_UPDATE:
        if ( typeid(*Dataset) == typeid(wsdair__SQLDatasetType) ) {
          wsdair__SQLDatasetType* SQLDataset = (wsdair__SQLDatasetType*) Dataset;
          SQLDataset->wsdair__SQLUpdateCount.push_back(num_column_or_row);
        }
        return SOAP_OK;
      default:
        //print ERROR
        return wrongDran(soap, dran);
    } 
  } else {             // SELECT
    switch(opType) {
      case OP_ITEM:
      case OP_ROWSET:
      case OP_TUPLE:
        {
          DBResult dbResult;
          vector<string> results;
          ret = handleRequest(table, count, num_column_or_row, dbResult, results);
          if( ret != SOAP_OK ) return ret;

          ret = makeResponse(num_column_or_row, results);
          if( ret != SOAP_OK ) return ret;
          break;
        }
      default:
        {
          return wrongDran(soap, dran);
          break;
        }
    }
  }
  return SOAP_OK;
}

void S_SQLResponse::init( struct soap *soap
                 , AutoDBConnection* helper
                 , unsigned int position
                 , unsigned int* reqCount
                 , std::string& dran
                 , wsdai__DatasetType* Dataset
                 , int opType
                 , bool _debug)
{
  this->soap = soap;
  this->position = position;
  this->reqCount = reqCount;
  this->dran = dran;
  this->Dataset = Dataset;
  this->opType = opType;
  this->helper = helper;
  overFlow = false;
  debug = _debug;
}

int S_SQLResponse::ValidDran()
{
  EntryPropList propList;

  int ret = checkDBConnection(soap, helper);
  if (ret != SOAP_OK ) return ret;

  int error = 0;
  do {
    ret = helper->dbConn->dbServer->getEntryProps(dran, propList);
    if( helper->dbConn->dbServer->handleEntryPropErrors(ret, dran) ) {
      DMESG("Error on retrieving EntryProps \n");
      error = 1; break;
    }

    EntryProps p(*propList.begin());

    if( p.flags == 0 ) { 
      DMESG("Deleted Table \n");
      error = 1; break; 
    }

    std::list< std::string > groups;
    if ( helper->dbConn->dbServer->checkPermissions(p, 'w', groups) <= 0) {
      return generateSoapError(soap, "Not Authorized Fault", NULL
              , SOAP_TYPE_wsdai__NotAuthorizedFaultType );
    }

  } while (0);

  DBResult dbResult;
  std::vector<string> results;
  helper->dbConn->getResult(dbResult, results);
  if (dbResult.errorCode != 0) {   // error
    return wrongDran(soap, dran);
  }

  if ( error ) { return wrongDran(soap,dran); }

  return SOAP_OK;
}

int S_SQLResponse::DestroyDataResource(std::string &table, char type)
{
  // initialize query execution environment
  DBResult dbResult;
  std::vector<string> results;

  int ret = checkDBConnection(soap, helper);
  if (ret != SOAP_OK ) return ret;

  int error = 0;
  do {
    Statement statement(*(helper->dbConn->dbServer->dbConn), debug);

    if( statement.beginTransaction()) { error = 1; break; }
    if ( helper->dbConn->dbServer->removeTableFromIndex(dran
                    , "masterindex", statement))
    {  DMESG("Remove Table Error\n"); error = 1; break; }

    if ( type == 'S' ) {
      std::stringstream query_s ;
      query_s << "DROP VIEW " << table << "; ";
      if( statement.exec(query_s.str()))
        { DMESG("Drop View\n"); error = 1; break; }
    }
    statement.commitTransaction();
  } while (0) ;

  if(error) return wrongDran(soap, dran);

  helper->dbConn->getResult(dbResult, results);
  if (dbResult.errorCode != 0) {   // error
    return wrongDran(soap, dran);
  }

  return SOAP_OK;
}


void S_SQLResponseFactory::makeResponse(wsdai__DataResourceAddressListType* res)
{
  wsdai__DataResourceAddressType *er = makeEndPoint(soap, dran, "SQLResponse");
  res->wsdai__DataResourceAddress.push_back(( wsdai__DataResourceAddressType*) er);
}



