/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "S_SQLRowset.h"
#include "util/Statement.h"

static bool debug = false;
#include "debug.h"

using namespace std;

_wrs__metadata* S_SQLRowset::createMetadata(int colNum, std::string &table)
{
  _wrs__metadata* metadata = soap_new__wrs__metadata(soap,-1);
  metadata->column_count = StringOf(colNum) ;
  metadata->__sizecolumn_definition = colNum ;
  metadata->column_definition
    = soap_new__wrs__metadata_column_definition(soap ,colNum);

  DBResult dbResult;
  vector<string> results;
  handleRequest(table, 1, colNum, dbResult, results);

  vector<string>::const_iterator it = results.begin();
  for(int i=0;i<colNum;i++) {
    metadata->column_definition[i].column_index = StringOf(i+1);
    metadata->column_definition[i].auto_increment = "false";
    metadata->column_definition[i].case_sensitive = "true";
    metadata->column_definition[i].currency = "false";
    metadata->column_definition[i].nullable = "1";
    metadata->column_definition[i].signed_ = "true";
    metadata->column_definition[i].searchable = "true";

    std::string tmp = *it;
    size_t pos = tmp.find('.');
    if ( pos == string::npos ) {
      metadata->column_definition[i].column_label = tmp ;
      metadata->column_definition[i].column_name = tmp ;
    } else {
      metadata->column_definition[i].column_label
        = metadata->column_definition[i].column_name
        = tmp.substr(pos+1);
      metadata->column_definition[i].table_name
        = tmp.substr(0,pos);
    }
    it++;
  }
  return metadata;
}

void S_SQLRowset::makePropertyDocument(wsdai__PropertyDocumentType* doc)
{
  /* wsdai__PropertyDocumentType */
  std::string endpoint;
  std::string address;
  transEndPoint(soap, address, endpoint, "SQLResponse", dran);
  doc->wsdai__DataResourceAbstractName = endpoint;
  doc->wsdai__DataResourceManagement
     = _wsdai__DataResourceManagement__ExternallyManaged;

  wsdai__DatasetMapType *dmt = soap_new_wsdai__DatasetMapType(soap,-1);
  dmt->wsdai__MessageQName = "wsdair:GetTuples" ;
  dmt->wsdai__DatasetFormatURI = WRS_FORMAT_URI ;
  doc->wsdai__DatasetMap.push_back(dmt);

  _wsdai__DataResourceDescription *drd
     = soap_new__wsdai__DataResourceDescription(soap,-1);
  doc->wsdai__DataResourceDescription = drd;
  drd->__mixed = soap_strdup(soap, "AMGA");

  doc->wsdai__Readable = true;
  doc->wsdai__Writeable = true;
  doc->wsdai__ConcurrentAccess = true;
  doc->wsdai__TransactionInitiation = _wsdai__TransactionInitiation__Automatic;
  doc->wsdai__TransactionIsolation = _wsdai__TransactionIsolation__ReadCommitted;
  doc->wsdai__ChildSensitiveToParent = _wsdai__ChildSensitiveToParent__Sensitive;
  doc->wsdai__ParentSensitiveToChild = _wsdai__ParentSensitiveToChild__Sensitive;


  /* Language Map */
  wsdai__LanguageMapType *lmt = soap_new_wsdai__LanguageMapType(soap,-1);
  lmt->wsdai__MessageQName = "wsdair:SQLExecute";
  lmt->wsdai__LanguageURI = "http://www.sql.org/sql-92";
  doc->wsdai__LanguageMap.push_back(lmt);
  lmt = soap_new_wsdai__LanguageMapType(soap,-1);
  lmt->wsdai__MessageQName = "wsdai:GenericQuery";
  lmt->wsdai__LanguageURI = "http://cern.ch/amga";
  doc->wsdai__LanguageMap.push_back(lmt);

}

int S_SQLRowset::CountRows(std::string &table, int &rows) {
  DBResult dbResult;
  std::vector<string> results;
  int count;

  int ret = checkDBConnection(soap, helper);
  if (ret != SOAP_OK ) return ret;

  Statement statement(*(helper->dbConn->dbServer->dbConn), debug);

  std::string query = "SELECT COUNT(*) FROM " ;
  query.append(table) ;

  if ( statement.exec(query) ) 
    return servBusy(soap, "DB error");
  
  statement.bind(1, &count);

  if(statement.fetch()){
    count = 0;
  } else {
    rows = count;
  }

  statement.unBind();
  statement.close();

  return SOAP_OK;
}

int S_SQLRowset::makePropertyDocument(int num_column
                                    , wsdair__SQLRowsetPropertyDocumentType* doc
                                    , std::string &table)
{
  makePropertyDocument(doc);

  /* wsdair__SQLRowsetPropertyDocumentType */
  doc->wsdair__RowSchema = soap_new_wsdair__RowSchemaType(soap,-1);
  doc->wsdair__RowSchema->wrs__metadata = createMetadata(num_column, table);
  doc->wsdair__AccessMode = _wsdair__AccessMode__Random;
  return CountRows(table, doc->wsdair__NoOfRows) ;
}

