/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "soapHelpers.h"
#include "util/generators.h"
#include <iostream>
#include <sstream>
#include <assert.h>

static bool debug = false;
#define logstream std::cout
#include "debug.h"

using namespace std;

void processIterativeResponse(Metadata& md, ns1__Handler *handler) {
	DMESG("Processing response...\n");
  if (handler == NULL) {
    cout << "No handler received from server.";
  }
  int emptyAnswers = 0;
  int lines;
  int iterations = 1;
  if (handler->dataChunk == NULL) {
        cout << "DataChunk is NULL\n";
  }

  if (debug) {
    printVector(&(handler->dataChunk->data));
  }

  lines = handler->dataChunk->data.size();
  if (handler->dataChunk->last) {
    cout << "Lines received: " << lines << ", iterations: " << iterations << endl;
  }
  _ns1__GetNextResponse r;
  _ns1__getNext req;
  req.handle = handler->handle;
  while (true) {
//      cout << "Calling getNext. Handle: " <<  handler->handle << endl;
    DMESG("Calling getNext\n");
        iterations++;
    if (md.__ns1__getNext(&req, &r) != SOAP_OK) {
//              cout << "Soap fault." << endl;
      soap_print_fault(md.soap, stderr); // print error
      return;
    }
//      cout << "Ok." << endl;
    if (debug) {
      printVector(&(r.dataChunk->data));
    }
//      cout << "r.dataChunk: " << r.dataChunk << endl;
//      cout << "r.dataChunk->last: " << r.dataChunk->last << endl;

    lines += r.dataChunk->data.size();
    r.dataChunk->data.clear();
    if (r.dataChunk->last) {
                cout << "Lines received: " << lines << ", iterations: " << iterations << ", Empty packets: " << emptyAnswers<< endl;
                return;
    }
  }
}



//void populate(Metadata &md, const string &collection) {
//  std::vector <std::string> generators;
//  generators.resize(4);
//  generators[0]="pythia";
//  generators[1]="lepto";
//  generators[2]="herwig";
//  generators[3]="disent";
//
//  vector<string> keys;
//  vector<string> types;
//  keys.push_back("eventGen");
//  types.push_back("varchar(20)");
//
//  keys.push_back("events");
//  types.push_back("int");
//
//  keys.push_back("pt_max_jet");
//  types.push_back("float");
//
//  keys.push_back("l1");
//  types.push_back("integer");
//
//  keys.push_back("sin");
//  types.push_back("float");
//  _ns1__EmptyResponse dummyResponse;
//
//  _ns1__addEntry addEntryReq;
//  addEntryReq.entry = "/testBig";
//  addEntryReq.type = "collection";
//  ev_Soap(md.__ns1__addEntry(&addEntryReq, &dummyResponse));
//
//  _ns1__addAttr addAttrReq;
//  addAttrReq.collection = "/testBig";
//  addAttrReq.keys = &keys;
//  addAttrReq.types = &types;
//  ev_Soap(md.__ns1__addAttr(&addAttrReq, &dummyResponse));
//
//  vector<string> values;
//
//  for(int i=1; i<=10000; i++){
//    ostringstream file;
//    file << "/testBig/t" << i;
//    int ev=int(100000.0*rand()/RAND_MAX+10000.0);
//    ostringstream ev_s, pt_s, l1_s, s_s;
//    ev_s << ev;
//    std::string gen=generators[i%4];
//    float pt=int(100.0*rand()/RAND_MAX+10.);
//    pt_s << pt;
//    int l1=i%2;
//    l1_s << l1;
//    float s=sin((float)i);
//    s_s << s;
//
//    values.clear();
//    values.push_back(gen);
//    values.push_back(ev_s.str());
//    values.push_back(pt_s.str());
//    values.push_back(l1_s.str());
//    values.push_back(s_s.str());
//
//    _ns1__setAttr setAttrReq;
//    setAttrReq.entry = file.str();
//    setAttrReq.keys = &keys;
//    setAttrReq.values = &values;
//    ev_Soap(md.__ns1__setAttr(&setAttrReq, &dummyResponse));
//  }
//}
