/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef __FLEX_LEXER_H
#include "FlexLexer.h"
#endif

#ifndef __AMGA_SQL_H
#define __AMGA_SQL_H

#include <map>
#include "server/MDServer.h"

#define __MESG(str) { std::cout << str << "\n"; }


#define ER_NO_ERROR   		    0 
#define ER_NOT_VALID_TABLE_NAME    -1
#define ER_NO_ENTRY_IN_TABLE       -2
#define ER_PERMISSION_NOT_ALLOWED  -3
#define ER_FAIL_GET_COLUMN_NAME    -4
#define ER_NOT_VALID_TABLE_NAME2   -5

#define MAX_SQL_STR 1024000
#define USER_STRING "$!%@^#"

typedef struct {
  std::string table;
  std::string dbname;
  bool inSelection;
  int flags;
  std::string alias;
} dbTable;

typedef std::pair <int, std::string> TAttribute;
typedef std::pair <int, dbTable> TTable;

class AmgaSqlParser: public yyFlexLexer {
 public:
   int __sql_set_query(char* str, int type);
   void flush();
   int yylex();
   void processPlainTable(int plain, char* str);
   char* getVal() { return mylval; }
   char* parseResult;
   MDServer *mdserver;

   /* Internal Use */
   /* however declared in public, why ? hmm... */ 
   int error;
   char __sql_array[MAX_SQL_STR];
   char* __sql_buf ; 

   /* Count the number of column for doBulkRead() */
   /*    retrieve user given attribute name          */
   int countColumn ;
   unsigned int numColumn;
   void initCountColumn() ;
   void startCountColumn();
   void stopCountColumn() ;
   bool countingColumn();


   /* if a query targets a plain file, 
      then the following isPlain variable is set to 1 */
   int isPlain;

   /* if a query contains FILE field then hasFILE is set to 1 */
   int hasFILE;
   int hasPERM;
   int hasOWNER;
   int hasGROUP;

   /* cache all the information about tables used in a query  */
   int numUsedTable;
   std::map <int, dbTable> usedTable;
   /* cache attributes infos */
   std::map <int, std::string> usedAttr;
   void initUsedTable();
   void insertUsedTable(std::string table, std::string dbname, int flags);

   /* store table names included in the from_clause */
   /* In case of SELECT : FROM clause               */
   /* In case of INSERT : INTO clause               */
   /* In case of UPDATE :                           */
   /* In case of DELETE : FROM clause               */
   bool isFromTable;
   int numFromTable;
   std::map <int, dbTable> fromTable;
   void initFromClause();
   void startFromClause();
   void stopFromClause();
   void destroyFromClause();
   void insertFromClause(std::string table, 
                         std::string dbname, 
                         std::string alias , 
                         int flags);


   /* remember first met table name  */
   std::string firstReferTable;
   int firstRefered ;

   /* remember latestly met table name and attribute name */
   std::string tmpReferTable;
   std::string tmpReferAttr;
   int tmpReferFlags;

   /* query type : SELECT, UPDATE, DELETE, UPDATE */
   int sqlType;
   int tmpSqlType;

   /* sticky bit is set ? */
   int sticky;


 private:
   /* Internal Use */
   char* mylval;
   std::string extra;

};


#endif
