/*
  Copyright (c) Members of the EGEE Collaboration. 2004.
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

/*
 * AT&T lex can't handle this lexer due to lex bugs.  It works with flex
 * 2.3.7, pclex 2.0.5, and MKS lex 3.1a.
 */

%{

#include "debug.h"
#include "amga_sql_yacc.hpp"
#include <string.h>

#define yyerror sqlyyerror

#include "amga_sql.h"

#undef YY_INPUT
#define YY_INPUT(buf,result,max_size)                   \
        {                                               \
                AmgaSqlParser* asp = (AmgaSqlParser*) this; \
                int c = *(asp->__sql_buf)++;                   \
                if ( c == 0 )  { result =  YY_NULL; }      	\
                else if ( c == '\n' ) { result =  YY_NULL;  }   \
                else if ( c == EOF ) { result = YY_NULL;    }   \
                else { buf[0] = (char) c;  result = 1 ;     }   \
        }



%}
	/* MKS needs the next line to increase the NFA table */

VALID [A-Za-z][A-Za-z0-9_-]*

DOT2 ".."


%e 1200
%option c++ yyclass="AmgaSqlParser"
%option prefix="sqlyy"
%option noyywrap
%x strbegin

%%

	/* literal keyword tokens */

ADA		{ mylval = strdup( YYText() ); return ADA; }
ALL		{ mylval = strdup( YYText() ); return ALL; }
AND		{ mylval = strdup( YYText() ); return AND; }
ARRAY_UPPER	{ mylval = strdup( YYText() ); return AMMSC2; }
AVG		{ mylval = strdup( YYText() ); return AMMSC; }
MIN		{ mylval = strdup( YYText() ); return AMMSC; }
MAX		{ mylval = strdup( YYText() ); return AMMSC; }
SUM		{ mylval = strdup( YYText() ); return AMMSC; }
COUNT		{ mylval = strdup( YYText() ); return AMMSC; }
ANY		{ mylval = strdup( YYText() ); return ANY; }
AS		{ mylval = strdup( YYText() ); return AS; }
ASC		{ mylval = strdup( YYText() ); return ASC; }
AUTHORIZATION	{ mylval = strdup( YYText() ); return AUTHORIZATION; }
BETWEEN		{ mylval = strdup( YYText() ); return BETWEEN; }
BY		{ mylval = strdup( YYText() ); return BY; }
C		{ mylval = strdup( YYText() ); return C; }
CHAR(ACTER)?	{ mylval = strdup( YYText() ); return CHARACTER; }
CHECK		{ mylval = strdup( YYText() ); return CHECK; }
CLOSE		{ mylval = strdup( YYText() ); return CLOSE; }
COBOL		{ mylval = strdup( YYText() ); return COBOL; }
COMMIT		{ mylval = strdup( YYText() ); return COMMIT; }
CONTINUE	{ mylval = strdup( YYText() ); return CONTINUE; }
CREATE		{ mylval = strdup( YYText() ); return CREATE; }
CURRENT		{ mylval = strdup( YYText() ); return CURRENT; }
CURSOR		{ mylval = strdup( YYText() ); return CURSOR; }
DECIMAL		{ mylval = strdup( YYText() ); return DECIMAL; }
DECLARE		{ mylval = strdup( YYText() ); return DECLARE; }
DEFAULT		{ mylval = strdup( YYText() ); return DEFAULT; }
DELETE		{ mylval = strdup( YYText() ); return DELETE; }
DESC		{ mylval = strdup( YYText() ); return DESC; }
DISTINCT	{ mylval = strdup( YYText() ); return DISTINCT; }
DOUBLE		{ mylval = strdup( YYText() ); return DOUBLE; }
ESCAPE		{ mylval = strdup( YYText() ); return ESCAPE; }
EXISTS		{ mylval = strdup( YYText() ); return EXISTS; }
FETCH		{ mylval = strdup( YYText() ); return FETCH; }
FLOAT		{ mylval = strdup( YYText() ); return FLOAT; }
FOR		{ mylval = strdup( YYText() ); return FOR; }
FOREIGN		{ mylval = strdup( YYText() ); return FOREIGN; }
FORTRAN		{ mylval = strdup( YYText() ); return FORTRAN; }
FOUND		{ mylval = strdup( YYText() ); return FOUND; }
FROM		{ mylval = strdup( YYText() ); return FROM; }
FULL		{ mylval = strdup( YYText() ); return FULL; }
GENERATE_SERIES	{ mylval = strdup( YYText() ); return AMMSC2; }
GO[ \t]*TO	{ mylval = strdup( YYText() ); return GOTO; }
GRANT		{ mylval = strdup( YYText() ); return GRANT; }
GROUP		{ mylval = strdup( YYText() ); return GROUP; }
HAVING		{ mylval = strdup( YYText() ); return HAVING; }
IN		{ mylval = strdup( YYText() ); return IN; }
INDEX		{ mylval = strdup( YYText() ); return INDEX; }
INDICATOR	{ mylval = strdup( YYText() ); return INDICATOR; }
INNER		{ mylval = strdup( YYText() ); return INNER; }
INSERT		{ mylval = strdup( YYText() ); return INSERT; }
INT(EGER)?	{ mylval = strdup( YYText() ); return INTEGER; }
INTO		{ mylval = strdup( YYText() ); return INTO; }
IS		{ mylval = strdup( YYText() ); return IS; }
JOIN		{ mylval = strdup( YYText() ); return JOIN; }
KEY		{ mylval = strdup( YYText() ); return KEY; }
LANGUAGE	{ mylval = strdup( YYText() ); return LANGUAGE; }
LEFT		{ mylval = strdup( YYText() ); return LEFT; }
LIKE		{ mylval = strdup( YYText() ); return LIKE; }
LIMIT		{ mylval = strdup( YYText() ); return LIMIT; }
MODULE		{ mylval = strdup( YYText() ); return MODULE; }
NATURAL		{ mylval = strdup( YYText() ); return NATURAL; }
NOT		{ mylval = strdup( YYText() ); return NOT; }
NULL		{ mylval = strdup( YYText() ); return NULLX; }
NUMERIC		{ mylval = strdup( YYText() ); return NUMERIC; }
OF		{ mylval = strdup( YYText() ); return OF; }
OFFSET		{ mylval = strdup( YYText() ); return OFFSET; }
ON		{ mylval = strdup( YYText() ); return ON; }
OPEN		{ mylval = strdup( YYText() ); return OPEN; }
OPTION		{ mylval = strdup( YYText() ); return OPTION; }
OR		{ mylval = strdup( YYText() ); return OR; }
ORDER		{ mylval = strdup( YYText() ); return ORDER; }
OUTER		{ mylval = strdup( YYText() ); return OUTER; }
PASCAL		{ mylval = strdup( YYText() ); return PASCAL; }
PLI		{ mylval = strdup( YYText() ); return PLI; }
PRECISION	{ mylval = strdup( YYText() ); return PRECISION; }
PRIMARY		{ mylval = strdup( YYText() ); return PRIMARY; }
PRIVILEGES	{ mylval = strdup( YYText() ); return PRIVILEGES; }
PROCEDURE	{ mylval = strdup( YYText() ); return PROCEDURE; }
PUBLIC		{ mylval = strdup( YYText() ); return PUBLIC; }
REAL		{ mylval = strdup( YYText() ); return REAL; }
REFERENCES	{ mylval = strdup( YYText() ); return REFERENCES; }
RIGHT		{ mylval = strdup( YYText() ); return RIGHT; }
ROLLBACK	{ mylval = strdup( YYText() ); return ROLLBACK; }
SCHEMA		{ mylval = strdup( YYText() ); return SCHEMA; }
SELECT		{ mylval = strdup( YYText() ); return SELECT; }
SET		{ mylval = strdup( YYText() ); return SET; }
SMALLINT	{ mylval = strdup( YYText() ); return SMALLINT; }
SOME		{ mylval = strdup( YYText() ); return SOME; }
SQLCODE		{ mylval = strdup( YYText() ); return SQLCODE; }
TABLE		{ mylval = strdup( YYText() ); return TABLE; }
TO		{ mylval = strdup( YYText() ); return TO; }
UNION		{ mylval = strdup( YYText() ); return UNION; }
UNIQUE		{ mylval = strdup( YYText() ); return UNIQUE; }
UPDATE		{ mylval = strdup( YYText() ); return UPDATE; }
USER		{ mylval = strdup( YYText() ); return USER; }
USING		{ mylval = strdup( YYText() ); return USING; }
VALUES		{ mylval = strdup( YYText() ); return VALUES; }
VIEW		{ mylval = strdup( YYText() ); return VIEW; }
WHENEVER	{ mylval = strdup( YYText() ); return WHENEVER; }
WHERE		{ mylval = strdup( YYText() ); return WHERE; }
WITH		{ mylval = strdup( YYText() ); return WITH; }
WORK		{ mylval = strdup( YYText() ); return WORK; }

"=" 		{ mylval = strdup("="); return EQUAL; }
"<>" 		{ mylval = strdup("<>"); return COMPARISON; }
"<"		{ mylval = strdup("<"); return COMPARISON; }
">"		{ mylval = strdup(">"); return COMPARISON; }
"<="		{ mylval = strdup("<="); return COMPARISON; }
">="		{ mylval = strdup(">="); return COMPARISON; }

{VALID}		{ mylval = strdup( YYText() ); return NAME; }

[-+*/:(),.;&|\[\]]	{ const char* temp = YYText() ;  mylval = strdup(temp); return temp[0]; }

	/* table name */

("/"{VALID})+ 		|
{VALID}("/"{VALID})* 	|
"."("/"{VALID})*		|
{DOT2}("/"{DOT2})*("/"{VALID})*		{ mylval = strdup( YYText() ); return TNAME; }

	/* numbers */

[0-9]+	{ mylval = strdup( YYText() ); return INTNUM ; }
[0-9]+"."[0-9]* 	|
"."[0-9]+ 	{ mylval = strdup( YYText() ); return INTNUM; }

[0-9]+[eE][+-]?[0-9]+	|
[0-9]*"."[0-9]*[eE][+-]?[0-9]+ { mylval = strdup( YYText() ); return APPROXNUM; }

        /* delimited literal */

\"      {
                char temp[MAX_SQL_STR];
                int j = 1;
                int c = yyinput();
                temp[0] = '\"';

                while( c != 0 && c != -1) {
                  temp[j] = c;
                  j++;
                  if(c == '\"'){
                    c = yyinput();
                    if( c == '\"') {
                      temp[j] = c;
                      j++;
                    } else {
		      unput(c);
                      temp[j] = 0;
                      mylval = strdup(temp);
                      return NAME;
                    }
                  }
                  c = yyinput();
                }

                __MESG("Unterminated Delemeter");
         }


\'              { extra = "'"; BEGIN(strbegin); }
<strbegin>\'\'       { extra.append("''"); }
<strbegin>\'         { BEGIN(INITIAL); 
                  extra.append("'");
                  mylval = strdup(extra.c_str());
                  return STRING; }
<strbegin>.          { extra.append(YYText()); }

		


[ \t\r]+	;	/* white space */

%%

void AmgaSqlParser::flush() {
 yy_flush_buffer(YY_CURRENT_BUFFER);
}


/*************************************************************
*  __sql_set_query initialzes a parser while setting a query 
*  to execute.
**************************************************************/

int AmgaSqlParser::__sql_set_query(char* str, int type)
{

  /* lex and yacc temparay variable initialization */
  int size = strlen(str);
  if ( size > MAX_SQL_STR - 1 || size < 1 ) return -1;
  memset(__sql_array, 0, MAX_SQL_STR);
  memcpy(__sql_array, str, size);
  __sql_buf = __sql_array;


  /* init variables used to store parsed result  */
  parseResult = NULL ;
  error = ER_NO_ERROR;

  /* init variables used to count columns */ 
  numColumn = 0;
  initCountColumn();


  /* init variables used to cache table names and attributes names */
  initUsedTable();
  if (! usedAttr.empty())  usedAttr.clear();

  /* query type */
  sqlType = type;
  tmpSqlType = type;

  /* table type is plain ? Initially No*/
  isPlain = 0 ;

  /* a query contains FILE attribute ? and etc...*/
  hasFILE = 0 ;
  hasPERM = 0 ;
  hasOWNER = 0 ;
  hasGROUP = 0 ;

  /* initialization needed for FromClause */
  initFromClause();

  firstRefered = 1 ;


  /* init sticky bit, initailly no */
  sticky = 0;

  return size;
}


/************************************************
* initUsedTable initializes variables to store
*   all the information on tables that
*     the given query accesses
*************************************************/
void AmgaSqlParser::initUsedTable()
{
  numUsedTable = 0;
  if (! usedTable.empty()) usedTable.clear();
}

/************************************************
* insertUsedTable store information on one table 
*************************************************/
void AmgaSqlParser::insertUsedTable( std::string table, std::string dbname, int flags)
{
  dbTable dbt;
  dbt.table = table;
  dbt.dbname = dbname;
  dbt.flags = flags;
  numUsedTable++;
  usedTable.insert( TTable( numUsedTable, dbt) );
}


/************************************************
* initCountColumn, startCountColumn enables counting 
* stopCountColumn diables counting
* countingColumn returns whether it is counting or not
*************************************************/
void AmgaSqlParser::initCountColumn()  { countColumn = 0; }
void AmgaSqlParser::startCountColumn() { countColumn = 0; }
void AmgaSqlParser::stopCountColumn()  { countColumn = 1; }
bool AmgaSqlParser::countingColumn()   
{ 
  if (countColumn == 0) { return true; } 
  else { return false; }
}


/************************************************
* initFromClause initializes variables to store
*   all the tables names that are located in
*     FROM clause.
*************************************************/
void AmgaSqlParser::initFromClause()
{
  isFromTable = false;
  numFromTable = 0;
  fromTable.clear();
}
void AmgaSqlParser::startFromClause() { isFromTable = true; }
void AmgaSqlParser::stopFromClause() { isFromTable = false; }
void AmgaSqlParser::destroyFromClause()
{
  numFromTable = 0;
  fromTable.clear();
}
void AmgaSqlParser::insertFromClause(std::string table, std::string dbname,
                                     std::string alias, int flags )
{
  if( isFromTable == false ) return ;
  dbTable dbt;
  dbt.table = table;
  dbt.dbname = dbname;
  dbt.alias = alias;
  dbt.flags = flags;
  numFromTable++;
  fromTable.insert( TTable( numFromTable, dbt) );
}


/*****************************************************
* Parsed table name has the form of "$    :AAAA$" 
* processPlainTable replaces "$...." to proper form 
*   depending on whether the target table is plain or not. 
******************************************************/
void AmgaSqlParser::processPlainTable(int plain, char* str){
   char* strtmp = str;
   int count = 0;
   int has_array = 0 ;

/*
   while ( *strtmp != 0 ) {
     if( *strtmp == '$' ) 
	count++;
     strtmp++;
   }

   if( (count % 2) == 1 ) {
     plain = 1;
   }
*/

   strtmp = str;
   
   if( plain == 1 ) {
     while ( *strtmp != 0 ) {
       if( strncmp(strtmp,USER_STRING,6) == 0 ) { 
         do {
           *strtmp = *(strtmp+6);
           strtmp++;
         } while ( *(strtmp+6) != '$' ) ;

	 *strtmp++ = ' '; 
	 *strtmp++ = ' '; 
	 *strtmp++ = ' '; 
	 *strtmp++ = ' '; 
	 *strtmp++ = ' '; 
	 *strtmp++ = ' '; 
	 *strtmp++ = ' '; 
       } else {
         strtmp++;
       }
     }
   } else {
     while ( *strtmp != 0 ) {
       if( strncmp(strtmp,USER_STRING,6) == 0 ) { 
	 *strtmp = '\"';  strtmp++ ;
	 *strtmp = 'u';  strtmp++ ;
	 *strtmp = 's';  strtmp++ ;
	 *strtmp = 'e';  strtmp++ ;
	 *strtmp = 'r';  strtmp++ ;
	 *strtmp = ':';  strtmp++ ;
         while( *strtmp != '$' ) {
           strtmp++;
         }
	 *strtmp = '\"';  strtmp++ ;
       } else {
         strtmp++;
       }
     }
   }
}


