/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "Clock.h"
#include <unistd.h>
#include <iostream>
#include <stdexcept>
#include <sstream>

using namespace std;

double Clock::CLK_SEC = sysconf(_SC_CLK_TCK);

Clock::Clock() 
: state(Stopped)
{
  reset();
}

Clock::~Clock()
{
}

void Clock::checkIsInState(State allowedState) {
  if (state != allowedState) {
    ostringstream os;
    os << "Invalid state: " << state << ". Should be: " << allowedState;
    throw runtime_error(os.str());
  }
}

void Clock::reset() {
  checkIsInState(Stopped);
  acc_time = 0;
  acc_cpu.tms_utime = 0;
  acc_cpu.tms_stime = 0;  
}

void Clock::start() {
  checkIsInState(Stopped);
  st_time = times(&st_cpu);
  state = Running;
}


void Clock::stop() {
  checkIsInState(Running);
  struct tms cpu;
  clock_t en_time = times(&cpu);    
  acc_time += en_time - st_time;
  acc_cpu.tms_utime += cpu.tms_utime - st_cpu.tms_utime;
  acc_cpu.tms_stime += cpu.tms_stime - st_cpu.tms_stime;
  state = Stopped;
}

void Clock::resume() {
  checkIsInState(Stopped);
  st_time = times(&st_cpu);
  state = Running;
}

double Clock::getRealTime() {
  return (acc_time)/CLK_SEC;
}

string Clock::print(const std::string &msg) {
    ostringstream os;
    os << msg << ": Real: " << (acc_time)/CLK_SEC 
       << " User: " << (acc_cpu.tms_utime)/CLK_SEC
       << " System: " << (acc_cpu.tms_stime)/CLK_SEC << std::endl;
    return os.str();
}
