/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "DBUtil.h"

#include "Statement.h"

using namespace std;

SQLException::SQLException(const std::string &message) throw ()
  : userMessage(message)  {
}

SQLException::SQLException(Statement &statement) throw () {
    userMessage.append("Error executing statement: ");
    userMessage.append(statement.getError());
}

SQLException::SQLException(const std::string &message, Statement &statement) throw () {
    userMessage.append(message);
    userMessage.append(":");
    userMessage.append(statement.getError());
}

const char *SQLException::what() const throw() {
    return userMessage.c_str();
};


StatementWrapper::StatementWrapper(Statement &statement) 
: _statement(statement)
{
}

StatementWrapper::~StatementWrapper()
{
}


void StatementWrapper::exec(const string &query) 
throw (SQLException) 
{
    if (_statement.exec(query)) {
      if (_statement.getErrorCode() != SQL_NO_DATA) {        
        throw SQLException(_statement);
      }
    }
}
