/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: Display.cpp,v 1.6 2010/06/22 09:23:01 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Display.h"
#include <iostream>

pthread_mutex_t Display::mutex = PTHREAD_MUTEX_INITIALIZER;
std::ostream Display::outstream(std::cout.rdbuf());
std::ostream Display::errstream(std::cerr.rdbuf());
int Display::maxLogLen = -1;

void Display::out(const std::string& str) {
	pthread_mutex_lock(&Display::mutex);
	if( maxLogLen > 0 && str.size() > maxLogLen ) {
          str.resize(maxLogLen);
          outstream << str << "[...]\n";
        } else {
	  outstream << str;
        }
	outstream.flush();
	pthread_mutex_unlock(&Display::mutex);
}

void Display::reset(std::ostream& os) {
	pthread_mutex_lock(&Display::mutex);
	os.flush();
        os.clear();
        os.seekp(0);
	pthread_mutex_unlock(&Display::mutex);
}


void Display::err(const std::string& str) {
	pthread_mutex_lock(&Display::mutex);
	errstream << str;
	errstream.flush();
	pthread_mutex_unlock(&Display::mutex);
}


std::streambuf *Display::setOutput(std::streambuf *newstream)
{
  pthread_mutex_lock(&Display::mutex);
  std::streambuf *oldstream = outstream.rdbuf(newstream);
  pthread_mutex_unlock(&Display::mutex);
  return oldstream;
}


std::streambuf *Display::setErrorOutput(std::streambuf *newstream)
{
  pthread_mutex_lock(&Display::mutex);
  std::streambuf *oldstream = errstream.rdbuf(newstream);
  pthread_mutex_unlock(&Display::mutex);
  return oldstream;
}
