/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// Prevents ostream collisions.
#ifndef DISPLAY_H
#define DISPLAY_H
#include <string>
#include <pthread.h>
#include <iostream>

/**
	Thread-safe output. Should be used when multiple threads are
	writing to stdout. The out() method will acquired a global
	lock, making sure that each write operation completes without
	interference from other threads.

	This method won't work for synchronizing among multiple processes
	since mutexs are per-process.
*/
class Display {
	static pthread_mutex_t mutex;
	static std::ostream outstream;
	static std::ostream errstream;
	static int maxLogLen;  
  public:
	static void out(const std::string& str);
	static void reset(std::ostream& os);
	static void err(const std::string& str);
	static std::streambuf *setOutput(std::streambuf *newstream);
	static std::streambuf *setErrorOutput(std::streambuf *newstream);
	static void setMaxLogLen(int l) { maxLogLen = l; };
};
#endif // DISPLAY_H ///:~
