/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: Path.cc,v 1.9 2010/04/14 07:15:30 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Path.h"
#include <assert.h>
#include <iostream>

//! Transforms a relative path into an absolute one
/**
 * Directories given as arguments must end with a /
 */
std::string absolutePath(const std::string &p,
			 const std::string &currentDir)
{
  if(!p.size()) return "";

  std::string path = p;
  if(path[0]!='/'){
    path=currentDir; // Current dir ends in '/' !
    path.append(p);
  }

  bool didSomething;
  do {
    didSomething=false;
    // handle "//" -> "/"
    size_t pos=path.find("//");
    if(pos!=std::string::npos){
      didSomething=true;
      path=path.substr(0, pos).append(path.substr(pos+1));
    }

    // handle /xxx/../yyy
    pos=path.find("/..");
    if(pos!=std::string::npos){
      didSomething=true;
      if(pos!=0){
	size_t pos2=path.rfind("/", pos-1);
	assert(pos2!=std::string::npos);
	path=path.substr(0, pos2).append(path.substr(pos+3));
	if(!path.size()) path="/";
      } else {
	if(path.size() > 3)   // not "/.."
	  path=path.substr(3);
	else
	  path="/";
      }
    }

    // handle /xxx/./yyy
    pos=path.find("/./");
    if(pos!=std::string::npos){
      didSomething=true;
      path=path.substr(0, pos).append(path.substr(pos+2));
    }
  } while(didSomething);

  if(path.size()> 2  // case "/xxx/." -> "/xxx/"
     && path[path.size()-1]=='.' && path[path.size()-2]=='/')
    path=path.substr(0, path.size()-2);

  if(path.size()== 2   // case "/." -> "/"
     && path[path.size()-1]=='.' && path[path.size()-2]=='/')
    path=path.substr(0, path.size()-1);

  if(path.size()> 1 && path[path.size()-1]=='/')
    path=path.substr(0, path.size()-1);

  return path;
}


std::string addEndSlash(const std::string &directory) {
  if (directory.size() == 0) {
    return "";
  } else {
    if (directory[directory.size()-1] != '/') {
      return directory + "/";
    } else {
      return directory;
    }
  }
}


std::string inSameDir(const std::string &knownPath,
		      const std::string &unknownPath)
{
  size_t n = knownPath.size();
  n = unknownPath.size() < n? unknownPath.size() : n;
  
  size_t i;
  for( i = 0; i < n; i++ ) {
    if( knownPath[i] != unknownPath[i] )
      break;
  }

  size_t lastSlash = unknownPath.rfind("/");
  if( lastSlash == std::string::npos )
    return "";

  if( lastSlash + 2 > unknownPath.size() )
    return "";

  if( lastSlash < i )
    return unknownPath.substr(lastSlash+1);

  return "";
}
