/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#include "SocketUtil.h"
#include <sstream>

#include <unistd.h> 

#include "debug.h"
static bool debug = true;

using namespace std;


void throwException(const string& msg) throw (runtime_error) {
    ostringstream oss;
    oss << msg  << ": " << strerror(errno);
    throw runtime_error(oss.str());
}


void write(int socket, const string& text) throw (runtime_error) {
    int bytes= write(socket, text.c_str(), text.size());
    if(bytes < (int)text.size()) {
        throwException("Could not write to socket");
    }
    DMESG("out> " << text <<  "<" << std::endl);
}

void write(int socket, char c) throw (runtime_error) {
    int bytes= write(socket, &c, 1);
    if(bytes < 1) {
        throwException("Could not write to socket");
    }
    DMESG("out> " << c <<  "<" << std::endl);
}


//! Reads at least a line and removes the training "\n"
/** Reads a line from an SSL or normal network connection using a
    buffer to handle consecutive lines. At least a line is returned
    (blocks!) and the trainling "\n" removed.
*/ 
void readLine(int ipConnection, std::string &line, std::string &buffer) throw (runtime_error)
{
    
    int bytes;
    size_t pos=buffer.find_first_of("\n");
    if (pos!=std::string::npos) {
        line=buffer.substr(0, pos);
        buffer=buffer.substr(pos+1);
        DMESG("in >" << line << "<" << std::endl);
        return;
    }
    
    while (buffer.find_first_of("\n")==std::string::npos) {
        char buff[1001];
        if((bytes=read(ipConnection, buff, 1000)) > 0){
            buff[bytes]='\0';
            buffer.append(buff);
        }
    }
    
    if (bytes < 0) { // Close connection
        throwException("Error reading");
    }
    if (bytes==0) {
        throw runtime_error("Connection reset by peer");
    }
    readLine(ipConnection, line, buffer);
}
