/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: Utils.cpp,v 1.19 2010/06/28 02:51:54 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Utils.h"

#include <sstream>
#include <iostream>
#include <time.h>
#include <errno.h>
#include <stdio.h>

#include "debug.h"
static bool debug = true;

using namespace std;

static char hextable[] = "0123456789abcdef\0";

int toInteger(const std::string &str) throw (std::runtime_error) {
    istringstream in(str);
    int xid;
    in >> xid;
    return xid;
}

int64_t toInt64(const std::string &str) throw (std::runtime_error) {
    istringstream in(str);
    int64_t xid;
    in >> xid;
    return xid;
}

uint64_t toUint64(const std::string &str) throw (std::runtime_error) {
    istringstream in(str);
    uint64_t xid;
    in >> xid;
    return xid;
}

std::string toString(uint64_t value) {
  ostringstream oss;
  oss << value;
  return oss.str();  
}

std::string toString(uint32_t value) {
  ostringstream oss;
  oss << value;
  return oss.str();  
}

std::string toString(int64_t value) {
  ostringstream oss;
  oss << value;
  return oss.str();  
}  

std::string toString(int value) {
  ostringstream oss;
  oss << value;
  return oss.str();  
}


std::string makeTimestamp()
  throw (std::runtime_error)
{
  struct tm t;
  time_t now = time(NULL);
  char buffer[41];

  gmtime_r(&now, &t);
  // Format: 1999-12-08 04:05:06
  strftime(buffer, 40, "%F %T", &t);

  return buffer;
}


int decodeSI(const std::string &si, std::list< int > &sites)
{
  if( si.size()%2 )
    return -1;

  for( size_t i = 0; i < si.size()/2; i++ ) {
    char part[3];
    int byte;
    part[0] = si[i*2];
    part[1] = si[i*2+1];
    part[2] ='\0';
    sscanf(part, "%x", &byte);
    int bit = 1;
    for( int j = 0; j < 8; j++) {
      if( byte & bit ) {
	sites.push_back(i * 8 + j);
      }
      bit <<= 1;
    }
  }
  return 0;
}


int changeSIBit(std::string &si, unsigned int id, bool setBit)
{

  unsigned int byte = id/8;

  if( si.size() <= 2*byte+1 )
    si.resize(2*byte+2, '0');

  unsigned char oldByte;
  char part[3];
  part[0] = si[2*byte];
  part[1] = si[2*byte+1];
  part[2] = '\0';

  sscanf(part, "%hhx", &oldByte);

  if(setBit)
    oldByte |= 1 << id%8;
  else
    oldByte &= ~( 1<< (id%8));

  sprintf(part, "%02x", oldByte);
  si.replace(2*byte, 2, part);

  return 0;
}


int countSIBits(const std::string &si)
{
  char buffer[3];
  buffer[2]='\0';
  unsigned char byte, mask;
  int count = 0;

  for( size_t i = 0; i < si.size()/2; i++) {
    buffer[0]=si[i*2];
    buffer[1]=si[i*2+1];
    sscanf(buffer, "%hhx", &byte);
    for (mask = 0x80; mask != 0; mask >>= 1) {
      if (byte & mask)
	count++;
    }
  }
  return count;
}


std::string decodeBinarySI(unsigned char *b, int size)
{
  std::string s(size*2, ' ');
  for( int i = 0; i < size; i++ ){
    s[i*2] = hextable[(b[i] & 0xf0)/16];
    s[i*2+1] = hextable[b[i] & 0x0f];
  }
  return s;
}


int encodeBinarySI(unsigned char * buffer, const std::string &si)
{
  if( si.size()%2 != 0)
    return -1;
  
  if( si.size()/2 > 1025)
    return -1;

  char part[3];
  buffer[2] = '\0';
  for( size_t i=0; i < si.size(); i+=2) {
    part[0] = si[i];
    part[1] = si[i+1];

    sscanf(part, "%hhx", buffer + i/2);
  }
  
  return si.size()/2;
}



std::string getErrorMsg() {
  char logbuff[256]; 
  strcpy(logbuff, strerror_r(errno, logbuff, 255));
  return string(logbuff);
}


std::string printBinary(unsigned char *b, int size)
{
  std::stringstream s;
  
  for(int i = 0; i < size; i ++ )
    s << hextable[(b[i] & 0xf0)/16] << hextable[b[i] & 0x0f];

  return s.str();
}
