/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifndef UTILS_H_
#define UTILS_H_

#include <string>
#include <list>
#include <stdexcept>
#include <stdint.h>

/* 64 bit integer handling */
int toInteger(const std::string &str) throw (std::runtime_error);

int64_t toInt64(const std::string &str) throw (std::runtime_error);

uint64_t toUint64(const std::string &str) throw (std::runtime_error);

std::string toString(uint32_t value);
std::string toString(uint64_t value);
std::string toString(int value);
std::string toString(int64_t value);
  
/* Convert the error message stored in errno to a string*/
std::string getErrorMsg();

/* Timestamps */
std::string makeTimestamp() throw (std::runtime_error);

int decodeSI(const std::string &si, std::list< int > &sites);

int changeSIBit(std::string &si, unsigned int id, bool setBit);

std::string decodeBinarySI(unsigned char *b, int size);
int encodeBinarySI(unsigned char * buffer, const std::string &si);

int countSIBits(const std::string &si);

std::string printBinary(unsigned char *b, int size);

#endif /*UTILS_H_*/
