/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: amgaProtocol.cc,v 1.8 2010/04/14 07:15:30 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "amgaProtocol.h"

namespace AMGA {

/// Escape transfer control sequences in the command
void encodeLine(const std::string &in, std::string &out)
{
  size_t pos = 0;
  while( pos < in.size() ) {
    size_t npos = in.find_first_of("\r\n\\", pos);
    if( npos == std::string::npos )
      npos = in.size();
    out = in.substr(pos, npos-pos);
    if( npos != in.size() ) {
      switch( in[npos] ) {
      case '\\':
	out.append("\\\\");
	break;
      case '\n':
	out.append("\\n");
	break;
      case '\r':
	out.append("\\r");
	break;
      }
    }      
    pos = npos + 1;
  }
}


void encodeLine(std::string &line)
{
  size_t pos = 0;
  while( pos < line.size() ) {
    pos = line.find_first_of("\r\n\\", pos);
    if( pos == std::string::npos )
      break;
    switch( line[pos] ) {
    case '\\':
      line.replace(pos, 1, "\\\\");
      break;
    case '\n':
      line.replace(pos, 1, "\\n");
      break;
    case '\r':
      line.replace(pos, 1, "\\r");
      break;
    }      
    pos += 2;
  }
}


void decodeLine(const std::string &in, std::string &out)
{
  size_t pos = 0;
  while( pos < in.size() ) {
    size_t npos = in.find("\\", pos);
    if( npos == std::string::npos )
      npos = in.size();
    out = in.substr(pos, npos-pos);
    if( npos < in.size()-1 ) {
      switch( in[npos+1] ) {
      case '\\':
	out.append("\\");
	pos++;
	break;
      case 'n':
	out.append("\n");
	pos++;
	break;
      case 'r':
	out.append("\r");
	pos++;
	break;
      default:
	out.append("\\");
	break;
      }
    } else {
      if( npos == in.size()-1 )
	out.append("\\");
    }
    pos = npos + 1;
  }
}


void decodeLine(std::string &line)
{
  size_t pos = 0;
  while( pos < line.size() ) {
    pos = line.find("\\", pos);
    if( pos == std::string::npos || pos >= line.size()-1 )
      break;
    switch( line[pos+1] ) {
    case '\\':
      line.replace(pos, 2, "\\");
      break;
    case 'n':
      line.replace(pos, 2, "\n");
      break;
    case 'r':
      line.replace(pos, 2, "\r");
      break;
    }
    pos++;
  }
}

}
