/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
//
// $Id: guidHelper.cc,v 1.8 2010/04/14 07:15:30 asunil Exp $
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "guidHelper.h"

#include <ctype.h>

#include "debug.h"
static bool debug = false;

namespace AMGA {

/// Escape transfer control sequences in the command
std::string validateGUID(const std::string &guid)
  throw (std::runtime_error)
{
  uuid_t uuid;

  if( guid == "gen" ) {
    uuid_generate_time(uuid);
    return hexifyGUID(uuid);
  }

  if( guid.substr(0,3) == "mac" ) {
    uuid_generate_time(uuid);

    std::string mac(guid,3);
    mac = normalizeMAC(mac);
    std::string u = hexifyGUID(uuid);
    u.replace(20, 12, mac);
    return u;
  }

  if( guid.size() != 36 )
    throw std::runtime_error("Wrong GUID size");

  if( uuid_parse( guid.c_str(), uuid) )
    throw std::runtime_error("Cannot parse GUID");
  
  return hexifyGUID(uuid);
}


std::string hexifyGUID(uuid_t uuid) 
  throw (std::runtime_error)
{
  char buffer[37];
  uuid_unparse(uuid, buffer);
  DMESG("GUID:> "<< buffer << "<\n");
  buffer[8]='\0';
  buffer[13]='\0';
  buffer[18]='\0';
  buffer[23]='\0';
  std::string u(buffer);
  u += buffer + 9;
  u += buffer + 14;
  u += buffer + 19;
  u += buffer + 24;

  return u;
}


void GUIDtoRaw(const std::string &guid, unsigned char buffer[16])
  throw (std::runtime_error)
{
  if(uuid_parse(guid.c_str(), buffer))
    throw std::runtime_error("Cannot parse GUID");
}


std::string niceGUID(const std::string &guid)
  throw (std::runtime_error)
{
  if( guid.size()!= 32 )
    throw std::runtime_error("Illegal GUID shortform");

  std::string g(guid, 0, 8);
  g.append("-");
  g.append(guid.substr(8, 4));
  g.append("-");
  g.append(guid.substr(12, 4));
  g.append("-");
  g.append(guid.substr(16, 4));
  g.append("-");
  g.append(guid.substr(20));
  
  return g;
}


std::string normalizeMAC(const std::string &mac)
  throw (std::runtime_error)
{
  if( mac.size() != 16 && mac.size() != 12 )
    throw std::runtime_error("Illegal MAC address");
  
  std::string m = mac;
  
  if( mac.size() == 16 ) {
    if( mac[2] != ':' || mac[4] != ':' || mac[6] != ':' 
	|| mac[8] != ':' || mac[10] != ':' )
      throw std::runtime_error("Illegal MAC address");

    m.append(mac, 0, 2);
    m.append(mac, 3, 2);
    m.append(mac, 6, 2);
    m.append(mac, 9, 2);
    m.append(mac, 12, 2);
    m.append(mac, 15, 2);
  } else {
    m = mac;
  }

  for( size_t i = 0; i < m.size(); i++ ) {
    if( !isxdigit(m[i]) )
      throw std::runtime_error("Illegal MAC address");
  }
   
  return m;
}

}
