/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


// -*- Mode:c++ -*-
// $Id: sslHelpers.h,v 1.3 2010/04/14 07:15:30 asunil Exp $
//
#ifndef SSL_HELPERS_H
#define SSL_HELPERS_H

#include <openssl/ssl.h>
#include <openssl/evp.h>
#include <openssl/err.h>

#if OPENSSL_VERSION_NUMBER >= 0x00908000L
#define OPENSSL_CONST const
#else
#define OPENSSL_CONST
#endif

#include <string>

void sslPrintErrors();
int sslHandleError(SSL *ssl, int ret);

struct sslExData{
  bool allowProxy;
  bool verbose;
};

int certVerifyCallback(int ok, X509_STORE_CTX *store);
int certVerifyCallbackAllowProxy(int ok, X509_STORE_CTX *store);
int checkIssuerCallback(X509_STORE_CTX *ctx, X509 *cert, X509 *issuer);
int isProxyCert(X509 *cert);
int isProxyOf(X509 *child, X509 *parent);

std::string getCertSubjectName(X509 *cert, bool traditional = false);

std::string sessionToPrintable(unsigned char *sessionID, unsigned int len);

// Message signing
EVP_MD_CTX *initSignMessage();
int updateSignMessage(EVP_MD_CTX *ctx, const std::string &message);
std::string finalizeSignMessage(EVP_MD_CTX *ctx, EVP_PKEY *pkey);

// Message verification
EVP_MD_CTX *initVerifyMessage();
int updateVerifyMessage(EVP_MD_CTX *ctx, const std::string &message);
int finalizeVerifyMessage(EVP_MD_CTX *ctx, EVP_PKEY *pkey, const std::string &signature);

#endif // SSL_HELPERS_H
