/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/*  Client program for a metadata soap server
    Usage: mdsoapclient [-d] [-p port] [-m mode] server
    mode : D or I  (Direct or Indirect)
    Example: 
       $ sqlaccess -p 8833 -m D http://localhost 
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <cstdio>

#include "soap/SQLAccess.h"
#include "debug.h"

using namespace std;

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <unistd.h>
#include <getopt.h>

#include <readline/readline.h>
#include <readline/history.h>


void printUsage()
{
  std::cout << "Usage: wsdair_access_execute [-d] [-s host] [-p port] [-u path] [-r abstractName] " << std::endl;
  std::cout << "       -h                    help message \n";
  std::cout << "       -d                    debug on\n";
  std::cout << "       -s host               \n";
  std::cout << "       -p port               \n";
  std::cout << "       -u path               \n";
  std::cout << "       -r DataResourceAbstactName    \n";
  std::cout << "       -c configFile    \n";
}

SQLAccess *client = NULL;

int main(int argc, char *argv[])
{  

  bool debug = false;
  int optionIndex=0;
  string host = "" ;
  int port = 0;
  string dran = "Metadata";
  string config = "";
  string path = "";

  static struct option options[] = {
    {"help", 0, 0, 0},
    {"debug", 0, 0, 0},
    {"server", 1, 0, 0},
    {"port", 1, 0, 0},
    {"dran", 1, 0, 0},
    {"config", 1, 0, 0},
    {"uPath", 1, 0, 0},
    {0, 0, 0, 0}
  };

  while(1){
    int c=getopt_long(argc, argv, "dhs:p:u:r:c:", options, &optionIndex);

    if(c==-1)
      break;

    switch (c) {
    case 'h':
      printUsage();
      exit(0);
    case 'd':
      debug=true;
      break;
    case 's':
      host = optarg ;
      break;
    case 'p':
      port = atoi(optarg) ;
      break;
    case 'u':
      path = optarg;
      break;
    case 'r':
      dran = optarg;
      break;
    case 'c':
      config = optarg;
      break;
    default:
      std::cout << "Unknown option" << (char)c << std::endl;
      printUsage();
      exit(5);
    }
  }

  if(optind!=argc-1 && optind!=argc){
    printUsage();
    exit(5);
  }
  
  if(optind == argc-1) 
    host=(std::string) argv[optind];


  client = new SQLAccess(host,port,path,config,true,debug);

  char* line;

  using_history();


  int i =0;
  std::string prompt = "Query> ";
      
  while(1) {
    i++;
  
    line = readline(prompt.c_str());
    
    std::string command;
    if( line )
      command = line;
    else               // NULL returned means EOF
      command = "quit";

    // Remove leading space
    size_t i = 0;
    while(i < command.size() && isspace(command[i]))
      i++;

    if( i >= command.size() ) continue;

    command = command.substr(i);


    // DMESG("Command : " << command << endl);

    if( command.compare(0,4,"quit") == 0 ) {
      break;
    }

    int res;

    add_history(line);
    free(line);

    // Register CTRL-C handler:
    registerClient(client);

    res = client->SimpleExecute(dran, command);

    // Un-Register CTRL-C handler:
    registerClient(NULL);

  }
}


