/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


/*  Client program for a metadata soap server
    Usage: wsdair_destory [-d] [-p port] [-s server] [-c conf] DataResourceAbstractName
    Example: 
       $ wsdair_destroy -p 8833 -s localhost /SQLResponse/sqlresponse55_3
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <cstdio>

#include "soap/CoreDataAccess.h"
#include "debug.h"

using namespace std;

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <unistd.h>
#include <getopt.h>

#include <readline/readline.h>
#include <readline/history.h>


void printUsage()
{
  std::cout << "Usage: wsdair_destory [-d] [-s host] [-p port] [-c conf] "
            << "[-u path] DataResourceAbstractName " << std::endl;
  std::cout << "       -h                    help message \n";
  std::cout << "       -d                    debug on\n";
  std::cout << "       -s host               \n";
  std::cout << "       -p port               \n";
  std::cout << "       -u path               \n";
  std::cout << "       -c configFile         \n";
}

CoreDataAccess *client = NULL;

int main(int argc, char *argv[])
{  

  bool debug = false;
  int optionIndex=0;
  string host = "" ;
  int port = 0;
  string dran = "Metadata";
  string config = "";
  string path = "";
  
  static struct option options[] = {
    {"help", 0, 0, 0},
    {"debug", 0, 0, 0},
    {"server", 1, 0, 0},
    {"port", 1, 0, 0},
    {"dran", 1, 0, 0},
    {"upath", 1, 0, 0},
    {"config", 1, 0, 0},
    {0, 0, 0, 0}
  };

  while(1){
    int c=getopt_long(argc, argv, "dhs:p:u:c:", options, &optionIndex);

    if(c==-1)
      break;

    switch (c) {
    case 'h':
      printUsage();
      exit(0);
    case 'd':
      debug=true;
      break;
    case 's':
      host = optarg ;
      break;
    case 'p':
      port = atoi(optarg) ;
      break;
    case 'c':
      config = optarg;
      break;
    case 'u':
      path = optarg;
      break;
    default:
      std::cout << "Unknown option" << (char)c << std::endl;
      printUsage();
      exit(5);
    }
  }

  std::string line;
  for(int i = optind; i < argc; i++) {
    line.append(argv[i]).append(" ");
  }

  size_t pos = line.find_first_not_of(' ',0);
  if (pos == std::string::npos ) {
    cout << "Empty DataResourceAbstractName\n" ;
    exit(-1);
  }
  DMESG("Dran : " << line << endl);

  client = new CoreDataAccess(host,port,path,config,false,debug);

  int ret = client->SimpleExecuteDestroy(line);

  return ret;
}


