/*
  Copyright (c) Members of the EGEE Collaboration. 2004. 
  See http://www.eu-egee.org/partners/ for details on the copyright
  holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 

    http://www.apache.org/licenses/LICENSE-2.0 

  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
*/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <cstdio>

#include "soap/CoreResourceList.h"
#include "debug.h"

using namespace std;

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <unistd.h>
#include <getopt.h>

#include <readline/readline.h>
#include <readline/history.h>


void printUsage()
{
  std::cout << "Usage: wsdair_list [-d] [-s host] [-p port] [-P path]" 
            << " [-c conf_file] " << std::endl;
  std::cout << "       -h                    help message \n";
  std::cout << "       -d                    debug on\n";
  std::cout << "       -s host               \n";
  std::cout << "       -p port               \n";
  std::cout << "       -u path               \n";
  std::cout << "       -c configFile    \n";
}

CoreResourceList *client = NULL;

int main(int argc, char *argv[])
{  

  bool debug = false;
  int optionIndex=0;
  string host = "" ;
  int port = 0;
  string dran = "Metadata";
  string path = "";
  string config = "";

  static struct option options[] = {
    {"help", 0, 0, 0},
    {"debug", 0, 0, 0},
    {"server", 1, 0, 0},
    {"port", 1, 0, 0},
    {"uPath", 1, 0, 0},
    {"dran", 1, 0, 0},
    {"config", 1, 0, 0},
    {0, 0, 0, 0}
  };

  while(1){
    int c=getopt_long(argc, argv, "dhs:u:p:r:c:", options, &optionIndex);

    if(c==-1)
      break;

    switch (c) {
    case 'h':
      printUsage();
      exit(0);
    case 'd':
      debug=true;
      break;
    case 's':
      host = optarg ;
      break;
    case 'u':
      path = optarg ;
      break;
    case 'p':
      port = atoi(optarg) ;
      break;
    case 'r':
      dran = optarg;
      break;
    case 'c':
      config = optarg;
      break;
    default:
      std::cout << "Unknown option" << (char)c << std::endl;
      printUsage();
      exit(5);
    }
  }

  client = new CoreResourceList(host,port,path, config,false,debug);

  int ret = client->SimpleExecuteList();

  return ret;
}


