dnl @synopsis AC_CHECK_OPENSSL
dnl
dnl This macro checks to see if the openssl library and header files 
dnl are installed.
dnl
dnl If the openssl/ssh.h heaeder is found, HAVE_OPENSSL_SSL_H
dnl is defined and the OPENSSL_CFLAGS set accordingly. If libssl.so
dnl is found it sets OPENSSL_LIBS accordingly.

AC_DEFUN([AC_CHECK_OPENSSL],
[dnl

AC_ARG_WITH(openssl-prefix,[  --with-openssl-prefix=PFX   Place where openssl is installed (optional)],
            openssl_prefix="$withval", openssl_prefix="")
if test x$openssl_prefix != x; then
  OPENSSL_CFLAGS="-I$openssl_prefix/include"  
fi

dnl Search for openssl/ssl.h header file in the given location
CXXFLAGS_SAVE=$CXXFLAGS
CXXFLAGS="$OPENSSL_CFLAGS $CXXFLAGS"
AC_CHECK_HEADER([openssl/ssl.h], ,
                 [AC_MSG_ERROR(["Could not find openssl/ssl.h header file!"])])

HAVE_OPENSSL_SSL_H=1
AC_DEFINE(HAVE_OPENSSL_SSL_H, 1, [Tells gSOAP openssl is available.])
CXXFLAGS=$CXXFLAGS_SAVE

dnl Search for openssl libarary: libssl

LIBS_SAVE=$LIBS
OPENSSL_LIBS="-L$openssl_prefix/lib -lssl -lcrypto"
LIBS="$LIBS $OPENSSL_LIBS"

AC_CHECK_LIB(crypto, EVP_CipherInit, ,
	 AC_MSG_ERROR([No libcrypto found!]))
AC_CHECK_LIB(ssl, SSL_library_init, ,
	 AC_MSG_ERROR([No openssl library found!]))

LIBS=$LIBS_SAVE

AC_MSG_NOTICE([Setting OPENSSL_LIBS to $OPENSSL_LIBS])

AC_SUBST(HAVE_OPENSSL_SSL_H)
AC_SUBST(OPENSSL_LIBS)
AC_SUBST(OPENSSL_CFLAGS)

])dnl
